
file_id = "ARDUINO/IO_CONFIG"

--[[

●機能概要

Arduino デバイス デジタルポートの入出力モード設定値を取得する
また同時に、SPI が使用可能かどうかのフラグ値も取得する


●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

USE_SPI			SPI を使用しているかどうかの設定値(16進数2桁)      		     "01"
				0x01 の場合に SPIインターフェイス関連の Firmataコマンドを
				使用できます

DIO_CONFIG		Arduinoデジタルポートの入出力モード設定値(16進数(2*port数)桁)	"FFFF"
				対応するビットが "1" の場合に出力モードで、"0" の場合は
				入力モードであることを示す


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[DIGITAL_CONFIG_QUERY] [DIGITAL_CONFIG_REPLY] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

ATmega328 を使用した Arduino ボードの場合には、 digital I/O port は 0,1 の2 つで
それぞれ、port0:PORTD, port1:PORTB(bit#0-bit#5) + (PORTC bit#0-#1)に対応しています。
詳しくは、Arduino IDE に同梱されている pins_arduino.h ファイルを参照して下さい。

SPI を使用している場合にはデジタルピンの D10-D13 までが SPI用に設定されています。
詳しくは SensorControlModule.ino スケッチプログラムの setup() 関数を参照してください。

●変更履歴

2017/06/28	abs_agent 用に排他制御コール部分を変更

2014/04/22	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Arduino digital 設定値取得 QUERY 送信と REPLY 受信
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "55"
local firmata_reply = "56"
local send_data = sysex_start .. firmata_request .. sysex_end

----------------------------------------------------------------------
-- 排他制御開始
-- 同じ Arduino デバイスに対して、QUERY コマンドが複数同時実行される場合に備えています。
-- 別のArduino デバイスを操作する場合や、同じ Arduino デバイスに対して異なる FIRMATA REPLY
-- コマンドバイト値を取得する場合には排他制御の必要はありません
----------------------------------------------------------------------
-- local cstat,handle = critical_section_enter("Firmata" .. firmata_request,5000);
local cstat = critical_section_enter2("Firmata" .. firmata_request,5000);
if not cstat then
	log_msg("critical_section_enter() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- リクエストフレーム送信とリプライ受信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data,firmata_reply)

----------------------------------------------------------------------
-- 排他制御終了
----------------------------------------------------------------------
-- cstat = critical_section_leave(handle)
cstat = critical_section_leave2("Firmata" .. firmata_request)
if not cstat then
	log_msg("critical_section_leave() failed",file_id)
	error()
end
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- 受信した FIRMATA フレームの内容をリターンパラメータに格納
----------------------------------------------------------------------
local data = hex72_to_tbl(string.sub(rdata,5,6),1)
if not script_result(g_taskid,"USE_SPI",bit_tohex(data[1],2)) then error() end

data = hex72_to_tbl(string.sub(rdata,7,-3),2)
local config_val = ""
if data ~= nil then
	for key,val in ipairs(data) do
		config_val = bit_tohex(val,2) .. config_val
	end
 	if not script_result(g_taskid,"DIO_CONFIG",config_val) then error() end
end

