
--[[

●機能概要

FASTDB データレコードをグローバル共有メモリエリアにバックアップまたはリストアする。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
mode			動作モードを指定する。パラメータ省略時は "backup"になる。

　　			"backup"	バックアップをグローバル共有メモリエリアに作成

				"restore"	グローバル共有メモリエリアに保存済みのバックアップから FASTDBデータをロード

																"backup"

key				<FASTDBキー名> 
				*必須*

host			<バックアップ先コンピュータのIPアドレス又はホスト名>
				省略時はこのスクリプトを実行したコンピュータ上にバックアップが作成される。

adj_sec			リストア時にタイムスタンプ値を補正する。
				単位は秒で小数点以下３桁(ミリ秒)まで指定可能。	-10.003

●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
STATUS			実行結果ステータス
				"OK" またはエラーメッセージ						"OK"

				このスクリプトをコールする agent_fastdb プログラムは、リターンパラメータの内容が
				"OK" または "*WARNING*' で始まる文字列の場合は処理を続行して、その他の内容の場合は処理が中断される。

●備考

host パラメータを省略した場合にはスクリプトを実行したローカルコンピュータ上にバックアップが作成される。
host パラメータを指定した場合は、一時的にバックアップがスクリプトを実行したコンピュータ上に
作成された後、リモートの abs_agent に転送される。その後、ローカルに残ったバックアップは消去される。

リモートコンピュータへのバックアップは、バックアップ先コンピュータのグローバル共有メモリエリアに以下のチャンネル名で作成する
同時にグローバル共有データにはバックアップ日付(YYYY/MM/DD HH:MM:SS)をチャンネル名と同一のキー名で格納する。
	
	"$FASTDB_BACKUP$<ローカルホスト名>$<FASTDBキー名>"

ローカルコンピュータへのバックアップ時のチャンネル名(と共有データのキー名)は下記になる

	"$FASTDB_BACKUP$<FASTDBキー名>"

●変更履歴

2023/11/05	  ver1.2 adj_sec パラメータ追加

2023/09/18    ver1.1 レコードサイズが 0 の場合はバックアップ対象から外して警告メッセージ返す

2023/08/30    ver1.0 初版作成

								abs_agent copyright(c) 2023 All Blue System
]]

-----------------------------
-- 動作モード
-----------------------------
local mode = "backup"
if g_params["mode"] then
	mode = g_params["mode"]
end
if (mode ~= "backup") and (mode ~= "restore") then 
	script_result("STATUS","*ERROR* mode error, " .. mode)
	return
end

----------------------------------
-- バックアップ対象 FASTDBキー名
----------------------------------
local fastdb_key
if g_params["key"] then
	fastdb_key = g_params["key"]
else
	script_result("STATUS","*ERROR* key parameter error")
	return
end
script_result("FASTDBKey",fastdb_key)

-----------------------------
-- バックアップ先ホスト
-----------------------------
local host = "localhost"
if g_params["host"] then
	host = g_params["host"]
end
script_result("BackupHost",host)

-----------------------------
-- バックアップ・チャンネル名
-----------------------------
local shmem_ch
if host == "localhost" then
	shmem_ch = "$FASTDB_BACKUP$" .. fastdb_key
else
	shmem_ch = "$FASTDB_BACKUP$" .. g_hostname .. "$" .. fastdb_key
end
script_result("Channel",shmem_ch)

----------------------------------
-- バックアップ時のタイムスタンプ
----------------------------------
local timestamp = string.format("%4.4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",stat_check(now_datetime()))

------------------------------------------------------------
-- FASTDB データをグローバル共有メモリエリアにバックアップ
------------------------------------------------------------
if mode == "backup" then

	-- レコード数が 0 のキーはバックアップ対象にしない
	local keys,records = stat_check(fastdb_key_list())
	local records_arr = {}
	for i,v in ipairs(keys) do
		records_arr[v] = records[i]
	end
	if records_arr[fastdb_key] and (records_arr[fastdb_key] == 0) then
		script_result("STATUS","*WARNING* record does not exist, skipped. FASTDBKey=" .. fastdb_key)
		return
	end

	if host == "localhost" then
		if not set_shared_data(shmem_ch,"") then
			script_result("STATUS","*ERROR* set_shared_data() failed.")
			return
		end
		if not fastdb_backup_shmem(fastdb_key,shmem_ch) then
			script_result("STATUS","*ERROR* fastdb_backup_shmem() failed.")
			return
		end
		if not set_shared_data(shmem_ch,timestamp) then
			script_result("STATUS","*ERROR* set_shared_data() failed.")
			return
		end
	else
		if not fastdb_backup_shmem(fastdb_key,shmem_ch) then
			script_result("STATUS","*ERROR* fastdb_backup_shmem() failed.")
			return
		end
		if not set_net_data(shmem_ch,"",host) then
			script_result("STATUS","*ERROR* set_net_data() failed.")
			return
		end
		if not shmem_transfer(shmem_ch,host) then
			script_result("STATUS","*ERROR* shmem_transfer() failed.")
			return
		end
		if not set_net_data(shmem_ch,timestamp,host) then
			script_result("STATUS","*ERROR* set_net_data() failed.")
			return
		end
		if not shmem_remove(shmem_ch) then
			script_result("STATUS","*ERROR* shmem_remove() failed.")
			return
		end
	end
end 

----------------------------------------------------------------------------
-- グローバル共有メモリエリアにバックアップしたデータから FASTDB にリストア
----------------------------------------------------------------------------
if mode == "restore" then
	local stat
	if host == "localhost" then
		if g_params["adj_sec"] then
			stat = fastdb_restore_shmem(fastdb_key,shmem_ch,tonumber(g_params["adj_sec"]))
		else
			stat = fastdb_restore_shmem(fastdb_key,shmem_ch)
		end
		if not stat then
			script_result("STATUS","*ERROR* fastdb_restore_shmem() failed.")
			return
		end
	else
		local info = stat_check(script_exec2("OS/GET_IP",{}))
		local my_addr = info["IPAddress"]
		if not shmem_transfer(shmem_ch,my_addr,host) then
			script_result("STATUS","*ERROR* shmem_transfer() failed.")
			return
		end
		if g_params["adj_sec"] then
			stat = fastdb_restore_shmem(fastdb_key,shmem_ch,tonumber(g_params["adj_sec"]))
		else
			stat = fastdb_restore_shmem(fastdb_key,shmem_ch)
		end
		if not stat then
			script_result("STATUS","*ERROR* fastdb_restore_shmem() failed.")
			return
		end
		if not shmem_remove(shmem_ch) then
			script_result("STATUS","*ERROR* shmem_remove() failed.")
			return
		end
	end
end 

script_result("STATUS","OK")
