
--[[

●機能概要

MGCP ファームウエアを搭載した M5Stack/Stick-C LCD に文字列を表示する


●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		操作対象のMGCPデバイスノード名							"M5_N001"

data		LCD に表示する文字列									"Hello World!!これは試験です"
			長い文字列は自動的に折り返して表示する。
			このスクリプトでは全ての文字を全角文字フォントで表示する。

x			表示開始位置 X 座標
			省略時は "0" が設定される								"20"

y			表示開始位置 Y 座標
			省略時は "0" が設定される								"10"

wrap		長い文字列を表示する際に折り返し表示を行う
			指定するパラメータ値は "0" (無効) または "1" (有効)		"1"
			省略時は "1" (wrap有効)が設定される。

cls			文字列表示前に画面をクリア(文字背景色で塗りつぶし)する。
			指定するパラメータ値は "0" (無効) または "1" (有効)		"1"
			省略時は "1" (cls有効) が設定される。

fg			文字色のカラーコード番号。 								"15"
			カラーコード番号については MGCP デバイスの lcd_preset_color コマンドを参照の事

bg			文字背景色のカラーコード番号。 							"0"
			カラーコード番号については MGCP デバイスの lcd_preset_color コマンドを参照の事

size		文字を size 倍に拡大して表示。 							"1"
			省略時は "1"

font		フォント名を指定する。									"SHINONOME_FONTX"
			font は グローバル共有メモリエリアにロードされた FONTX2 形式の
			フォントデータを示すチャンネル名を指定する。フォントファイルは
			scripts/preload/013_FONTX/FONTX_LIB.lua ファイル中の fontx_init() でロードされる。
			デフォルト "AYU_FONTX"

ln			前回表示した文字列の終端で改行した位置に文字列を表示する。
			必要に応じて同時に cls パラメータを "1" または "0" に指定すること。
			パラメータ値は何でも構わない							"1"
			
dx			ダイレクトコマンドモードを使用する
			パラメータ値は何でも構わない							"1"


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

●備考

表示と同時にグローバル共有変数 MGCP_LCD_MESSAGE_<node> に "1" を設定する。
このグローバル変数は最新のメッセージが表示されたことを示すフラグとして利用可能で、
任意のタイミングでクリアしても表示には影響しない。

●変更履歴

2020/12/24	M5Stick-C デバイスで split パラメータを nil から 20 に変更
			半角 ASCII 表示時に表示可能文字数が増加したため。

2020/8/26	初版作成

abs_agent	copyright(c) All Blue System

]]


-------------------------
-- パラメータチェック
-------------------------
local p_node
local p_data
local p_x = 0
local p_y = 0
local p_wrap = true
local p_cls = true
local p_fg = 15
local p_bg = 0
local p_size = 1

if g_params["node"] then
	p_node = g_params["node"]
else
	error()
end

if g_params["data"] then
	p_data = g_params["data"]
else
	error()
end

if g_params["x"] then
	p_x = tonumber(g_params["x"])
end

if g_params["y"] then
	p_y = tonumber(g_params["y"])
end

if g_params["wrap"] then
	if g_params["wrap"] == "1" then
		p_wrap = true
	else
		p_wrap = false
	end
end

if g_params["cls"] then
	if g_params["cls"] == "1" then
		p_cls = true
	else
		p_cls = false
	end
end

if g_params["fg"] then
	p_fg = tonumber(g_params["fg"])
end

if g_params["bg"] then
	p_bg = tonumber(g_params["bg"])
end

if g_params["size"] then
	p_size = tonumber(g_params["size"])
end

local valid_dev_type = false
local dev_type = stat_check(get_shared_data("$TYPE_" .. p_node))
local scr_w,scr_h,p_font,p_split

if dev_type == "M5Stick" then
	valid_dev_type = true
	scr_w = 160
	scr_h = 80
	p_font = "SHINONOME_FONTX"
	p_split = 20
end

if dev_type == "M5Stack" then
	valid_dev_type = true
	scr_w = 320
	scr_h = 240
	p_font = "AYU_FONTX"
	p_split = 20
end

if not valid_dev_type then error() end

if g_params["font"] then
	p_font = g_params["font"]
end

if g_params["ln"] then
	if g_params["dx"] then
		dxcmd_lcd_jprintln(p_node, p_data, p_cls ,p_font ,scr_w ,scr_h ,3 ,p_size ,p_fg ,p_bg)
	else
		mgcp_lcd_jprintln(p_node, p_data, p_cls ,p_font ,scr_w ,scr_h ,p_split ,p_size ,p_fg ,p_bg)
	end
else
	if g_params["dx"] then
		dxcmd_lcd_jprint(p_node, p_data, p_x,p_y, p_wrap, p_cls ,p_font ,scr_w ,scr_h ,3 ,p_size ,p_fg ,p_bg)
	else
		mgcp_lcd_jprint(p_node, p_data, p_x,p_y, p_wrap, p_cls ,p_font ,scr_w ,scr_h ,p_split ,p_size ,p_fg ,p_bg)
	end
end

set_shared_data("MGCP_LCD_MESSAGE_" .. p_node ,"1")

