
--[[

●機能概要

MGCP ファームウエアを搭載した M5Stack/M5Stick-C デバイス LCD ディスプレイに文字列を表示する
タイムスタンプのタイトルを自動的に追加して、メッセージを表示する。
メッセージ表示前に以前に表示されていた内容はクリアされる。

デバイスが M5Stick-C の場合にはメッセージ表示と同時に LED をブリンク表示させる。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		操作対象のMGCPデバイスノード名							"STICK001"

data		LCD に表示する文字列									"Hello World!!これは試験です"
			長い文字列は自動的に折り返して表示する。
			このスクリプトでは全ての文字を全角文字フォントで表示する。


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

●備考

表示と同時にグローバル共有変数 MGCP_LCD_MESSAGE_<node> に "1" を設定する。
このグローバル変数は最新のメッセージが表示されたことを示すフラグとして利用可能で、
任意のタイミングでクリアしても表示には影響しない。

●変更履歴

2020/12/25	M5Stick-C デバイスで split パラメータを nil から 20 に変更
			半角 ASCII 表示時に表示可能文字数が増加したため。
			バックグランドカラーとフォントサイズを調整

2020/1/29	初版作成

abs_agent	copyright(c) All Blue System

]]


-------------------------
-- パラメータチェック
-------------------------
local p_node
local p_data

if g_params["node"] then
	p_node = g_params["node"]
else
	error()
end

if g_params["data"] then
	p_data = g_params["data"]
else
	error()
end

-- タイトル部分に表示するタイムスタンプを取得
local now = os.date "*t"
local wday = stat_check(day_of_week(now["year"],now["month"],now["day"]))
local weekstr
if wday == 1 then 
	weekstr = "日"
elseif wday == 2 then
	weekstr = "月"
elseif wday == 3 then
	weekstr = "火"
elseif wday == 4 then
	weekstr = "水"
elseif wday == 5 then
	weekstr = "木"
elseif wday == 6 then
	weekstr = "金"
elseif wday == 7 then
	weekstr = "土"
end
local timestamp = string.format("%2.2d:%2.2d:%2.2d %d/%d(%s)",now["hour"],now["min"],now["sec"],now["month"],now["day"],weekstr);

-- 表示対象の MGCP ノード名のデバイスタイプを検索する。
-- デバイスタイプは　ノードから定期的に送信される heartbeat イベントメッセージを元にグローバル共有変数に設定されている
local dev_type = stat_check(get_shared_data("$TYPE_" .. p_node))
local cmd_timeout = 3000

----------------------------
-- M5Stick-C
----------------------------
if dev_type == "M5Stick" then
	local split = 20
	local title_size = 1
	local title_font = "SHINONOME14_FONTX"
	local title_color = 10
	local body_size = 1
	local body_font = "SHINONOME_FONTX"
	local body_color = 15
	local bg_color = 1

	-- タイトルに四角マークとタイムスタンプを表示
	local cmds = {}
	cmds["command"] = "lcd_draw"
	cmds["clear"] = bg_color
	cmds["fill"] = 1
	cmds["rect"] = {0,0,8,14,title_color}
	stat_check(mgcp_command(p_node,g_json.encode(cmds),false,cmd_timeout))
	stat_check(mgcp_lcd_jprint(p_node,timestamp,16,0,false,false,title_font,nil,nil,split,title_size,title_color,bg_color))

	-- 本文を表示する
	stat_check(mgcp_lcd_jprintln(p_node,p_data,false,body_font,nil,nil,split,body_size,body_color,bg_color))

	-- LED(GPIO#10) を点滅させる
	--[[
	cmds = {}
	cmds["command"] = "gpio_blink"
	cmds["gpio"] = 10
	cmds["flag"] = "on"
	stat_check(mgcp_command(p_node,g_json.encode(cmds),false,cmd_timeout))
	]]

	--他からの利用を想定して、新規メッセージを表示したことを示すフラグをセット。
	stat_check(set_shared_data("MGCP_LCD_MESSAGE_" .. p_node ,"1"))
end

----------------------------
-- M5Stack
----------------------------
if dev_type == "M5Stack" then
	local split = 20
	local title_size = 2
	local title_font = "SHINONOME12_FONTX"
	local title_color = 10
	local body_size = 2
	local body_font = "SHINONOME_FONTX"
	local body_color = 15
	local bg_color = 1

	-- タイトルに四角マークとタイムスタンプを表示
	local cmds = {}
	cmds["command"] = "lcd_draw"
	cmds["clear"] = bg_color
	cmds["fill"] = 1
	cmds["rect"] = {0,0,18,26,title_color}
	stat_check(mgcp_command(p_node,g_json.encode(cmds),false,cmd_timeout))
	stat_check(mgcp_lcd_jprint(p_node,timestamp,40,4,false,false,title_font,nil,nil,split,title_size,title_color,bg_color))

	-- 本文を表示する。タイトル文字と本文の間隔を空けてバランスをとるため最終表示位置をずらす。
	stat_check(set_shared_data("MGCP_LCD_PREV_" .. p_node,"0,10,20,1")) -- set offset position for the body-text
	stat_check(mgcp_lcd_jprintln(p_node,p_data,false,body_font,nil,nil,split,body_size,body_color,bg_color))

	--他からの利用を想定して、新規メッセージを表示したことを示すフラグをセット。
	stat_check(set_shared_data("MGCP_LCD_MESSAGE_" .. p_node ,"1"))

end

