
--[[

●機能概要

MGCP ノードデバイスのコンフィギュレーション値の読み込み・設定

** 注意 **
対象となる MGCP デバイスが "Demo" ライセンスの場合にはこの機能は使用できません。

●リクエストパラメータ
---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
node            MGCP デバイスノード名                            "ESP32_N01"

mode			動作モードを指定する。パラメータ省略時は "load" モードになる。

　　			"download"  MGCPデバイスからコンフィギュレーションをロードしてグローバル共有データに保存

				"upload" グローバル共有データに保存された設定を MGCP デバイスに送信・設定する。

config			ノードのコンフィギュレーションが格納された JSON 文字列
				mode に "upload" を指定した時の必須パラメータで、"download" 指定時には不要。
				config パラメータ値の例は、下記の config リターンパラメータの項を参照してください


●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
Stauts			処理が成功した場合には "OK" が入る
				指定したサービスが提供されていない等で、サーバーから結果を正常に
				受け取れなかった場合にはエラーメッセージが入る
																		"OK"

config 			ノードのコンフィギュレーションが格納された JSON 文字列
				(mode が "download" を指定したときにのみ値が返る)

{"i2_pul_sda_":[1,1],"i2_sda_":[21,26],"g_change_det":"800000000",
  "g_pull_":[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
  "i2_clock_":[100000,100000],"g_sampling":60,"nodename":"ESP32_N01","mqtt_user":"","publish_gpio":16,
  "i2_scl_":[22,25],"i2_pul_scl_":[1,1],"mqtt_keepalive":120,"accept_mac":1,"i2_use_":[0,0],
   "mqtt_host":"192.168.100.14","cfg_mode_gpio":35,"heartbeat":30,"g_check_int":10,"mqtt_port":1883,
   "g_md_":[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0],
    "remote_cfg":1,"accept_all":1,"mqtt_client_id":"abs:aaaxxx:12345",
   "g_init_":[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
   "publish_ind":1,"logserver_host":"192.168.100.45","mqtt_password":""}


●備考

このスクリプトを使用するためには、MGCP ノード側で最低限 Wi-Fi接続と MQTT ブローカへの接続が
確立している必要があります。このため、最低限以下の設定項目についてはノード設置時に適切な値に
設定してください。これらの設定は MGCP デバイスをコンソール(コンフィギュレーション)モードで起動した後、
シリアルポート経由で設定できます。

最低限必要な設定項目

wifi_ssid
wifi_password
nodename
mqtt_client_id
mqtt_host
mqtt_port
mqtt_user (認証が必要な場合)
mqtt_password (認証が必要な場合)

JSON 形式の文字列でコンフィギュレーションキーとその値が格納される。


●変更履歴

2021/8/21	 JOB config 追加

2021/3/15	 PWM/SERVO config 追加

2020/7/29	 Direct Command Port 追加

2020/4/20	 Wi-Fi TX Power, Direct Command 追加

2020/4/11    SPI ClockSpeed の設定が抜けていたのを修正

2020/2/27	 NeoPixel 設定追加

2020/1/14	 M5Stick-C 設定追加

2019/2/23    I2C config 設定追加

2019/2/9     f_alt_mqtt_host[], f_alt_mqtt_flag 追加

2018/10/16   初版作成

copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
local node = g_params["node"]
if not node then
	log_msg("node parameter error",file_id)
	error()
end

local mode = "download"
if g_params["mode"] then
	mode = g_params["mode"]
end
if (mode ~= "download") and (mode ~= "upload") then error() end

if (g_params["config"] == nil) and (mode == "upload") then error() end

-------------------------------------------------------------------------------------
-- MGCP デバイスのNVS に保存しているコンフィギュレーション情報
-- キー名やタイプに間違った値を指定すると既存のコンフィギュレーションが壊れてしまうため、
-- デバイスのバージョンに合わせて最新のエントリを使用すること。
--
-- 対応 MGCP version: 1.4b01
-------------------------------------------------------------------------------------


g_mgcp_cfg_keys = {

	--------------------------------------------------------
	-- <Key>                <Type>      <Mode>  <Comment> --
	--------------------------------------------------------

--	{"master_password",		"string",	"w",	"マスターパスワード"},    (セキュリティのためアクセス禁止)
	{"wifi_ssid",			"string",	"w",	"Wi-Fi SSID"},
	{"wifi_password",		"string",	"w",	"Wi-Fi パスワード"},
	{"logserver_host",		"string",	"rw",	"ログメッセージ出力先ホスト名"},
	{"nodename",			"string",	"rw",	"デバイスノード名"},

	{"mqtt_client_id",		"string",	"rw",	"MQTT クライアントID"},
	{"mqtt_host",			"string",	"rw",	"MQTT ホスト名"},
	{"alt_mqtt_host",		"string",	"rw",	"MQTT ホスト名 バックアップ用"},
	{"mqtt_port",			"i32",		"rw",	"MQTT ポート番号"},
	{"mqtt_user",			"string",	"rw",	"MQTT 接続ユーザー名"},
	{"mqtt_password",		"string",	"rw",	"MQTT 接続パスワード"},
	{"mqtt_keepalive",		"i32",		"rw",	"MQTT KeepAliveTimer(秒)"},
	{"alt_mqtt_flag",		"u8",		"rw",	"MQTT ホストの通常とバックアップ切り替えフラグ"},

	{"accept_all",			"u8",		"rw",	"_ALL_ ノード名のリクエストを受け付ける"},
	{"accept_mac",			"u8",		"rw",	"<MAC_Address> ノード名のリクエストを受け付ける"},
	{"heartbeat",			"i32",		"rw",	"heartbeat イベント送信間隔(秒)"},
	{"publish_ind",			"u8",		"rw",	"PUBLISH パケット受信時に GPIO ポートに接続した LED をブリンクさせる"},
	{"publish_gpio",		"u8",		"rw",	"publish_ind 有効時に出力する GPIO ポート番号"},
	{"cfg_mode_gpio",		"u8",		"rw",	"起動時にコンフィギュレーションモードにするかどうかをチェックする GPIOポート番号"},
	{"remote_cfg",			"u8",		"rw",	"MGCP コマンド経由でコンフィギュレーションの操作を許可する"},
	{"tx_power",			"i8",		"rw",	"Wi-Fi TX Power"},
	{"dx_command",			"u8",		"rw",	"MGCP Direct Command"},
	{"dx_port",				"i32",		"rw",	"MGCP Direct Command ポート番号"},

	{"g_check_int",			"i32",		"rw",	"GPIO 変化検出間隔(ミリ秒)"},
	{"g_change_det",		"hex64",	"rw",	"GPIO 変化検出対象。ビット位置の値が 1 の GPIO<#bit> が検出対象となる"},
	{"g_sampling",			"i32",		"rw",	"gpio_sampling イベント送信間隔(秒)"},

	-- lcd_xxx は M5StickC デバイスのみ使用
	{"lcd_rot",				"u8",		"rw",	"LCD 画面の向きを表す番号を指定する。1:LANDSCAPE, 3:LANDSCAPE_FLIP"},
	{"lcd_light",			"u8",		"rw",	"LCD バックライトの明るさを指定する。0 から 7 までの整数値を指定する。"},

	{"neo_use",				"u8",		"rw",	"NeoPixel デバイスを使用する"},
	{"neo_gpio",			"u8",		"rw",	"NeoPixel を接続している GPIO ポート番号"},
	{"neo_count",			"u16",		"rw",	"NeoPixel LED数"},
	{"neo_bright",			"u8",		"rw",	"NeoPixel 明るさ"},
	{"neo_b_fade",			"u16",		"rw",	"ブリンク表示時のフェード効果ディレイ(ミリ秒)"},
	{"neo_b_width",			"u16",		"rw",	"ブリンク表示間隔(ミリ秒)"},
	{"neo_maxamps",			"u16",		"rw",	"NeoPixel で消費する最大電流制限値(mA)"},

	{"jb_mode",				"u8",		"rw",	"JOB 実行フラグ"},
	{"jb_cmddata",			"blob",		"rw",	"定期実行用 ダイレクトコマンド(バイナリデータ)"},
	{"jb_initdata",			"blob",		"rw",	"初期化用 JOB ダイレクトコマンド(バイナリデータ)"},
	{"jb_intvl",			"i32",		"rw",	"JOB 実行間隔(ms)"},

}

g_mgcp_gpio_num = 40	-- GPIOポート数

g_mgcp_i2c_num = 2		-- I2Cバス数

g_mgcp_spi_num = 2		-- SPIポート数

g_mgcp_pwm_num = 4		-- PWM チャンネル数

g_mgcp_cfg_arrs = {

	{"g_md_",			"u8",	g_mgcp_gpio_num,	"rw",	"GPIO モード"},
	{"g_pull_",			"u8",	g_mgcp_gpio_num,	"rw",	"GPIO プルアップ"},
	{"g_init_",			"u8",	g_mgcp_gpio_num,	"rw",	"GPIO 初期値"},

	{"i2_use_",			"u8",	g_mgcp_i2c_num,		"rw",	"I2C バス有効"},
	{"i2_sda_",			"u8",	g_mgcp_i2c_num,		"rw",	"SDA にアサインした GPIOポート番号"},
	{"i2_pul_sda_",		"u8",	g_mgcp_i2c_num,		"rw",	"SDA プルアップ有効"},
	{"i2_scl_",			"u8",	g_mgcp_i2c_num,		"rw",	"SCL にアサインした GPIOポート番号"},
	{"i2_pul_scl_",		"u8",	g_mgcp_i2c_num,		"rw",	"SCL プルアップ有効"},
	{"i2_clock_",		"i32",	g_mgcp_i2c_num,		"rw",	"I2C クロック周波数(Hz)"},

	{"spi_use_",		"u8",	g_mgcp_spi_num,		"rw",	"SPI ポート有効"},
	{"spi_miso_",		"i8",	g_mgcp_spi_num,		"rw",	"MISO にアサインした GPIOポート番号(-1は不使用を意味)"},
	{"spi_mosi_",		"i8",	g_mgcp_spi_num,		"rw",	"MOSI にアサインした GPIOポート番号(-1は不使用を意味)"},
	{"spi_clk_",		"i8",	g_mgcp_spi_num,		"rw",	"SCK にアサインした GPIOポート番号(-1は不使用を意味)"},
	{"spi_cs_",			"i8",	g_mgcp_spi_num,		"rw",	"CS にアサインした GPIOポート番号(-1は不使用を意味)"},
	{"spi_clock_",		"i32",	g_mgcp_spi_num,		"rw",	"SPI クロックスピード(Hz)"},
	{"spi_mode_",		"u8",	g_mgcp_spi_num,		"rw",	"SPI MODE番号"},

	{"pwm_use_",		"u8",	g_mgcp_pwm_num,		"rw",	"PWM チャンネル有効"},
	{"pwm_gpio_",		"i8",	g_mgcp_pwm_num,		"rw",	"PWM チャンネルにアサインした GPIOポート番号(-1は不使用を意味)"},
	{"pwm_reso_",		"u8",	g_mgcp_pwm_num,		"rw",	"PWM 分解能ビット数"},
	{"pwm_freq_",		"i32",	g_mgcp_pwm_num,		"rw",	"PWM 周波数(Hz)"},
	{"pwm_duty_",		"i32",	g_mgcp_pwm_num,		"rw",	"PWM duty値 0..(2^分解能ビット数-1)"},
	{"servo_use_",		"u8",	g_mgcp_pwm_num,		"rw",	"PWM チャンネルをSERVO用に使用する"},

}

local mgcp_timeout = 3000

-------------------------------------------------------------------
-- キー文字列とタイプを指定して NVS データを取得してテーブルに格納
-------------------------------------------------------------------
function cfg_load_general(tbl,key,cfgtype)
	local req_json = '{"command":"nvs_get","key":"' .. key .. '","type":"' .. cfgtype .. '"}'
	local stat,reply = mgcp_command(node,req_json,false,mgcp_timeout)
	if not stat then
		script_result("Status","*ERROR* mgcp_command() failed.")
		return false
	end
	local rpl_tbl = g_json.decode(reply)
	if rpl_tbl.result == "success" then
		tbl[key] = rpl_tbl.value
	else
		script_result("Status","*ERROR* nvs_get command failed.")
		return false
	end
	return true
end

-----------------------------------------------------------------------------------------------------
-- キー・プリフィックス文字列とタイプ、個数を指定して複数の NVS データを取得して配列をテーブルに格納
-----------------------------------------------------------------------------------------------------
function cfg_load_with_prefix(tbl,prefix,cfgtype,count)
	local req_json = '{"command":"nvs_get_arr","count":' .. tostring(count) .. ',"prefix":"' .. prefix .. '","type":"' .. cfgtype .. '"}'
	local stat,reply = mgcp_command(node,req_json,false,mgcp_timeout)
	if not stat then
		script_result("Status","*ERROR* mgcp_command() failed.")
		return false
	end
	rpl_tbl = g_json.decode(reply)
	if rpl_tbl.result == "success" then
		tbl[prefix] = rpl_tbl.value -- value 配列には count 分のコンフィギュレーションが格納されている
	else
		script_result("Status","*ERROR* nvs_get_arr command failed.")
		return false
	end
	return true
end

-------------------------------------------------------------------------------------
-- キー・プリフィックス文字列とタイプ、設定値の配列を指定して複数の NVS データを設定
-------------------------------------------------------------------------------------
function cfg_save_with_prefix(newcfg,prefix,cfgtype)
	if type(newcfg[prefix]) ~= "table" then return false end
	local req_tbl = {}
	req_tbl["command"] = "nvs_set_arr"
	req_tbl["type"] = cfgtype
	req_tbl["prefix"] = prefix
	req_tbl["value"] = newcfg[prefix]
	local req_json = g_json.encode(req_tbl)
	local stat,reply = mgcp_command(node,req_json,false,mgcp_timeout)
	if not stat then
		script_result("Status","*ERROR* mgcp_command() failed.")
		return false
	end
	local rpl_tbl = g_json.decode(reply)
	if rpl_tbl.result ~= "success" then
		script_result("Status","*ERROR* nvs_set_arr command failed.")
		return false
	end
	return true
end

------------------------------------------------------
-- キー文字列とタイプを指定して NVS データを書き込む
------------------------------------------------------
function cfg_save_general(key,newval,cfgtype)
	local req_tbl = {}
	req_tbl["command"] = "nvs_set"
	req_tbl["key"] = key
	req_tbl["type"] = cfgtype
	req_tbl["value"] = newval
	local req_json = g_json.encode(req_tbl)
	local stat,reply = mgcp_command(node,req_json,false,mgcp_timeout)
	if not stat then
		script_result("Status","*ERROR* mgcp_command() failed.")
		return false
	end
	local rpl_tbl = g_json.decode(reply)
	if rpl_tbl.result ~= "success" then
		script_result("Status","*ERROR* nvs_set command failed.")
		return false
	end
	return true
end

------------------------------------------------------------------------
-- パラメータで指定したコンフィギュレーションを MGCP デバイスに設定する
------------------------------------------------------------------------
if mode == "upload" then
	local new_cfg_tbl = g_json.decode(g_params["config"])
	for k,v in ipairs(g_mgcp_cfg_keys) do
		-- nvs_set コマンドが許可されていて、かつ config パラメータで指定されたキーだけを対象にする
		if new_cfg_tbl[ v[1] ] and ((v[3] == "w") or (v[3] == "rw")) then 
			if not cfg_save_general(v[1],new_cfg_tbl[ v[1] ],v[2]) then return end
		end
	end
	--------------------------------------------------------------------
	-- キー名 prefix と繰り返し個数の組み合わせで構成した設定項目を設定
	--------------------------------------------------------------------
	for k,v in ipairs(g_mgcp_cfg_arrs) do
		-- nvs_set_arr コマンドが許可されていて、かつ config パラメータで指定されたキーだけを対象にする
		if new_cfg_tbl[ v[1] ] and ((v[4] == "w") or (v[4] == "rw")) then
			if not cfg_save_with_prefix(new_cfg_tbl,v[1],v[2]) then return end
		end
	end
	script_result("Status","OK")
end

---------------------------------------------------------------------------------------------
-- MGCP デバイスから現在のコンフィギュレーションをロードした後、キーと値をテーブルに格納する
---------------------------------------------------------------------------------------------
if mode == "download" then
	local config_tbl = {}
	for k,v in ipairs(g_mgcp_cfg_keys) do
		if (v[3] == "r") or (v[3] == "rw") then	-- nvs_get コマンドが許可されているキーだけを対象にする
			if not cfg_load_general(config_tbl,v[1],v[2]) then return end
		end
	end
	--------------------------------------------------------------------
	-- キー名 prefix と繰り返し個数の組み合わせで構成した設定項目ロード
	--------------------------------------------------------------------
	for k,v in ipairs(g_mgcp_cfg_arrs) do
		-- nvs_get_arr コマンドが許可されているキーだけを対象にする
		if (v[4] == "r") or (v[4] == "rw") then
			if not cfg_load_with_prefix(config_tbl,v[1],v[2],v[3]) then return end
		end
	end
	--------------------------------------------------------------------------------------
	-- テーブル形式のコンフィギュレーション値を JSON に変換してリターンパラメータにセット
	--------------------------------------------------------------------------------------
	script_result("Status","OK")
	script_result("config",g_json.encode(config_tbl))
end

