
--[[

●機能概要

グローバル共有変数に格納されている GPS NMEA-0183 センテンスデータから
現在地の緯度と経度、データ取得時刻を求める。

●参照するグローバル共有データ

---------------------------------------------------------------------------------
キー値			値/値の例
---------------------------------------------------------------------------------
$GPRMC			GPSレシーバから受信した最新の NMEA-0183 RMC センテンスが格納されている
				"$GPRMC,050246,A,4254.1621,N,14135.6146,E,0.0,30.6,240716,9.3,W,A*31"

$GPGGA			GPSレシーバから受信した最新の NMEA-0183 GGA センテンスが格納されている
				"$GPGGA,050246,4254.1621,N,14135.6146,E,1,05,2.7,30.6,M,30.7,M,,*72"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
status			処理が成功した場合には "OK" が格納される
				エラーが発生した場合にはエラーメッセージが格納される	"OK"
lat				Degree 形式の緯度を設定									"35.680743"
lon				Degree 形式の経度を設定									"139.768914"
timestamp		GPS データ中のローカル日時を設定						"2016/02/01 23:22:10"

●備考

●変更履歴

2016/07/27  初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-----------------------------------------------------------------------
-- GPSレシーバから受信した NMEA-0183 センテンスが保存済みか
-----------------------------------------------------------------------
local stat,gps_rmc,gps_gga
stat,gps_rmc = get_shared_data("$GPRMC")
stat,gps_gga = get_shared_data("$GPGGA")
if (gps_rmc == "") or (gps_gga == "") then
	script_result(g_taskid,"status","GPSデータが保存されていません")
	return
end

-----------------------------------------------------------------------
-- NMEA センテンス中の使用するカラムをローカル変数にコピーする
-----------------------------------------------------------------------
local rmc_arr = csv_to_tbl(gps_rmc)
local gga_arr = csv_to_tbl(gps_gga)
local rmc_time = rmc_arr[2]
local rmc_stat = rmc_arr[3]
local rmc_lat = rmc_arr[4]
local rmc_lat_compass = rmc_arr[5]
local rmc_lon = rmc_arr[6]
local rmc_lon_compass = rmc_arr[7]
local rmc_date = rmc_arr[10]
local gga_alt = gga_arr[10]

-----------------------------------------------------------------------
-- GPS レシーバーステータスが有効な場合のみ処理を行う
-----------------------------------------------------------------------
if rmc_stat ~= "A" then
	script_result(g_taskid,"status","レシーバステータスが有効ではありません")
	return
end

-----------------------------------------------------------------------
-- GPSデータからローカル日時を計算
-----------------------------------------------------------------------
local local_date,local_time
stat, local_date, local_time = gps_utc_to_local(rmc_date,rmc_time)
if not stat then
	script_result(g_taskid,"status","GPSタイムスタンプ変換エラー")
	return
end
script_result(g_taskid,"timestamp",local_date .. " " .. local_time)

-----------------------------------------------------------------------
-- GPSデータのローカル時刻はサーバーと比べて10分以上ずれていないか
-----------------------------------------------------------------------
local now = os.date "*t"
local seconds, gps_year, gps_month, gps_day, gps_hour, gps_min, gps_sec
stat, gps_year, gps_month, gps_day, gps_hour, gps_min, gps_sec = str_to_datetime(local_date .. " " .. local_time)
stat, seconds = seconds_between(now["year"],now["month"],now["day"],now["hour"],now["min"],now["sec"],
								gps_year, gps_month, gps_day, gps_hour, gps_min, gps_sec)
if math.abs(seconds) > 600 then
	script_result(g_taskid,"status","最新のGPSデータを受信していません")
	return
end

-----------------------------------------------------------------------
-- GPSデータの座標単位を Degree形式に変換する
-----------------------------------------------------------------------
local lat_deg,lon_deg
stat,lat_deg,lon_deg = gps_coordinate_deg("dmm",rmc_lat,rmc_lon,rmc_lat_compass,rmc_lon_compass)
if not stat then
	script_result(g_taskid,"status","座標単位変換エラー")
	return
end
script_result(g_taskid,"lat",lat_deg)
script_result(g_taskid,"lon",lon_deg)
script_result(g_taskid,"status","OK")

