
--[[

●機能概要

ローカルまたはリモート XBee デバイスで AT コマンドを実行する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値	値				           								値の例
---------------------------------------------------------------------------------
com		XBee デバイスをローカルに接続しているシリアルポート名またはタイトル名
		省略した場合には abs_agent に登録済みのシリアルデバイスを検索して
		デバイスタイプが "XBEE" のものを自動的に選択する。このとき、シリアル
		デバイスが見つからなかった場合や、2 つ以上の "XBEE" デバイスが見つかった
		場合にはエラーとなる。
																"/dev/ttyUSB0"
atcmd	2 ASCII character(upper case)
		XBee API コマンドで指定可能な ATコマンド文字列。ASCII 大文字で２文字
																"NI"
dest
		送信先のXBeeデバイスを指定する。dest文字列には以下のいずれかを指定する。
		XBeeデバイスの Node Identification 文字列
		XBeeデバイスの Serial Number(64bitアドレス)16進数表記の文字列
		XBeeデバイスの Source Address(16bitアドレス)16進数表記の文字列
		"*" ブロードキャストアドレス
		パラメータ省略時や "" 空文字列指定時、abs_agent に接続したXBeeデバイスが対象
																"Device2"

param_hex (hexstr)
		XBee API コマンドパラメータを指定する場合に、データ(複数バイト)を16進数文字列で指定する。
		パラメータ省略時や "" 空文字列を指定すると、コマンドパラメータは送信されない。
		param_str パラメータと同時に指定してはいけない。
																"44657669636535"

param_str (text string)
		XBee API コマンドパラメータを指定する場合に、データ(複数バイト)をASCII文字列で指定する。
		パラメータ省略時や "" 空文字列を指定すると、コマンドパラメータは送信されない。
		param_hex パラメータと同時に指定してはいけない。
																"Device5"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値	値		            									値の例
---------------------------------------------------------------------------------
Stauts	処理が成功した場合には "OK" が入る
		指定したサービスが提供されていない等で、サーバーから結果を正常に
		受け取れなかった場合にはエラーメッセージが入る
		リプライフレーム中のステータスバイトの値が 0x00 以外の場合にも
		エラーメッセージが格納される。
																"OK"

com		ローカル XBee デバイスが接続されたシリアルポート名		"/dev/ttyUSB0"

data	リプライフレームからコマンドデータ部分を抜粋したもの。
		通常は16進数文字列で格納される。ただし atcmd が "NI" の場合には ASCII 文字列
		に変換したものが格納される。
		リプライフレーム中にコマンドデータが存在しなかった場合には data パラメータは
		返されない。

●備考

●変更履歴

2019/7/2	初版作成

abs_agent	copyright(c) 2019 All Blue System

]]

local atcmd
if g_params["atcmd"] and (g_params["atcmd"] ~= "") then
	atcmd = string.upper(g_params["atcmd"])
else
	script_result("Status","*ERROR* atcmd parameter not found.")
	return
end

--------------------------------------------------------------------
-- ローカル XBee デバイスが接続されたシリアルポート名を取得する
--------------------------------------------------------------------
local com
if g_params["com"] and (g_params["com"] ~= "") then
	com = g_params["com"]
else
	-- シリアルポートを検索して XBee デバイスタイプのポート名取得を試みる
	local xbee_port_count = 0
	local stat,port,title,dev_type,buff = serial_all_list()
	if not stat then error() end
	for key,val in ipairs(port) do
		if dev_type[key] == "XBEE" then
			xbee_port_count = xbee_port_count + 1
			com = val
		end
	end
	if xbee_port_count == 0 then
		script_result("Status","*ERROR* serial(type:XBEE) port not found.")
		return
	end
	if xbee_port_count > 1 then
		script_result("Status","*ERROR* multiple serial(type:XBEE) ports.")
		return
	end
end
script_result("com",com)

--------------------------------------------------------------------
-- AT コマンド実行対象のデバイスを選択する
--------------------------------------------------------------------
local dest = ""
if g_params["dest"] then
	dest = g_params["dest"]
end

--------------------------------------------------------------------
-- AT コマンドパラメータが指定されていた場合には 16進数文字列に揃える
--------------------------------------------------------------------
local param = ""
if g_params["param_hex"] and (g_params["param_hex"] ~= "") then
	param = g_params["param_hex"]
end
if g_params["param_str"] and (g_params["param_str"] ~= "") then
	param = tbl_to_hex(str_to_tbl(g_params["param_str"]))
end

--------------------------------------------------------------------
-- AT コマンド実行とリプライ受信
--------------------------------------------------------------------
local stat,frame,mframe = xbee_at_command(com,dest,atcmd,param)
if not stat then
	script_result("Status","*ERROR* xbee_at_command() failed")
	return
end

local data_arr = {}
local frame_arr = hex_to_tbl(frame)
-- for AT Command Response frame
if frame_arr[4] == 0x88 then
	if frame_arr[8] ~= 0x00 then
		script_result("Status","*ERROR* AT Command Response, Status = 0x" .. bit_tohex(frame_arr[8],2))
		return false
	end
	for i = 9,#frame_arr - 1,1 do
		table.insert(data_arr,frame_arr[i])
	end
end
-- for Remote Command Response frame
if frame_arr[4] == 0x97 then
	if frame_arr[18] ~= 0x00 then
		script_result("Status","*ERROR* Remote Command Response, Status = 0x" .. bit_tohex(frame_arr[18],2))
		return false
	end
	for i = 19,#frame_arr - 1,1 do
		table.insert(data_arr,frame_arr[i])
	end
end
if #data_arr > 0 then
	if atcmd == "NI" then
		script_result("data",tbl_to_str(data_arr))
	else
		script_result("data",tbl_to_hex(data_arr))
	end
end

script_result("Status","OK")
