
--[[

●概要

MQTT ブローカを利用したリクエスト->リプライ方式の実装。

JSON 形式で表現されたリクエストメッセージを MQTT ブローカに送信します。
MQTT ブローカに接続しているサービス提供側では、リクエストメッセージのトピックによって、
自身のデバイスで処理すべきかを判断(購読対象にする)してメッセージを取り込みます。

その後、メッセージ中のリクエストコマンドやパラメータを元に、処理を行った結果を再び
リプライメッセージとしてMQTT ブローカに送信します。

リクエストを行った側では、MQTT ブローカから送信されてくるリプライメッセージのトピックを
見て、サービス処理結果を受け取ります。

このスクリプトは、MQTT_PUBLISH イベントハンドラ中に記述されたリプライメッセージ情報を取得する
スクリプトと連動して動作します。

●セットアップ

(1) MQTTエンドポイント作成と abs_agent 再起動

このスクリプトを使用する前に、abs_agent の設定ファイルに MQTTブローカへのエンドポイントを作成しておきます。
設定ファイルを修正後は必ず abs_agent を再起動してください。

(2) MQTT PUBLISH イベントハンドラ中の MGCPプロトコル処理部分を有効にする

また、MQTT PUBLISH メッセージを受信した場合に実行される MQTT_PUBLISH イベントハンドラ中に、リプライメッセージ
受信時の処理を記述しておきます。詳しくは、MQTT_PUBLISH イベントハンドラ中の処理部分をご覧ください。インストール直後
はコメントになっていますのでコメント指定文字を削除してください。


●MQTTエンドポイント設定例

このスクリプトで使用している MQTT エンドポイントの設定例です。
abs_agent.xml 設定ファイル中の MQTT 設定に記述します。詳しいタグの説明は abs_agent ユーザーマニュアルを参照してください。

このライブラリ定義では、MQTT ブローカ間との通信時に指定する QoS はデフォルトで 0 を指定しています。
これは、この関数内でリプライメッセージを確認するため、メッセージ自身の送達レベルは最低限の 0 で十分と判断したためです。

<Title>,<ClientID> と <BrokerHostName> の内容は使用する MQTT ブローカに合わせて変えて下さい。

$HOSTNAME$ 部分はabs_agent の動作しているコンピュータのホスト名に変換されてから MQTT サービスモジュールが起動します。
もし、MGCP で使用する自身のノード名に別の名前を使用したいばあいには変更してください。


    <MQTT>
      <AutoOnline type="boolean">True</AutoOnline>
      <KeepAliveTimer type="integer">60</KeepAliveTimer>
      <EndPointList>
        <Item>
          <Title>リモートデバイス処理</Title>
          <ClientID>abs9k:12345-raspi</ClientID>
          <BrokerHostName>192.168.100.14</BrokerHostName>
          <PortNumber>1883</PortNumber>
          <AutoSubscribeTopicList>/request/$HOSTNAME$/#,/request/_ALL_/#,/reply/$HOSTNAME$/#,/event/$HOSTNAME$/#,/event/_ALL_/#</AutoSubscribeTopicList>
          <AutoSubscribeQoSList>0,0,0,0,0</AutoSubscribeQoSList>
          <UserName/>
          <Password/>
          <WillTopic/>
          <WillMessage/>
          <WillQoS>0</WillQoS>
          <WillRetain>False</WillRetain>
          <RecvBuffInit>2000</RecvBuffInit>
          <DetailLog>True</DetailLog>
        </Item>
      </EndPointList>
    </MQTT>


●MGCP(Megumino-Garden Communication Protocol)プロトコル説明

このプロトコルは主に LANや PAN 等の近距離に配置された多数のデバイス(ノード)を管理するために設計して
います。具体的には自宅や庭・ガレージ、工場、オフィス等の LAN や Wi-Fiに接続されたセンサーや
アクチュエータを想定しています。

これらのデバイスでは LAN または Wi-Fi 接続機能をもっていて、MQTT プロトコルで特定の MQTTブローカに
接続できることを想定しています。デバイスの数が増えた場合に複数の MQTT ブローカに分散して
デバイスを設置することもできます。

デバイス側にはセンサーや IO 装置が接続されていて、リモートからのリクエストに応じて操作すること
ができます。リクエストメッセージはデバイスが接続している MQTT ブローカ経由で配信されます。

リクエストメッセージに対応するリプライメッセージをデバイスからリクエスト元に送信することも
できます。この場合にも MQTT ブローカ経由でメッセージが届けられます。複数の MQTT ブローカに
デバイスを分散配置している場合には、リクエスト元のコンピュータはそれぞれのMQTTブローカへの
エンドポイントを確立している必要があります。

デバイス側で発生する各種イベントをブローカーに通知する機能も有しています。イベントには
定期的に送信する heartbeat や GPIO が変化したことを通知する gpio_change_detect 等があります。

ネットワーク接続には TCP/IP 上の MQTT プロトコルを使用していて、このプロトコルで定義された
ペイロード送信とトピック選択機能を使って MGCP プロトコルを実現しています。

MGCP プロトコルの特徴：

* デバイス側では MQTT プロトコルをサポートした TCP/IP 接続機能を利用する

* ネットワーク接続はデバイス側から MQTT ブローカ側への方向に接続されて、エラー発生の検出や
  再接続もデバイス側で自動的に行う。

* リクエスト元のデバイスやコンピュータは、操作対象のデバイスが接続されている MQTT ブローカに
  接続する。その後リクエストメッセージやイベントメッセージを格納したペイロードを特定のデバイス向けに
  送信する。このとき全デバイスを対象に送信することも可能。複数の MQTT ブローカを利用している
  場合に、デバイスが定期的にイベント送信している heartbeat メッセージを利用することで、デバイスの
  ノード名から収容されている MQTT ブローカのエンドポイントを自動で見つけられるので、リクエスト側は
  ノードのみを指定するだけで送信できる。

* デバイス側ではユーザー側で名前をつけたノード名やMAC アドレスを利用した、そのデバイス特有のトピックを
  常に購読していて、配信されたリクエストメッセージやイベントメッセージを選択して受信する。

* デバイス側からリプライメッセージをリクエスト元に返信する場合には、リクエストメッセージ
  受信時に指定されたトピック中のメッセージID 文字列を使用して、リプライメッセージ送信時のトピックに
  指定する。

* デバイス側で特定の条件になったときには、イベントの種別を含んだトピックを使用してイベントメッセージを送信する。


(1) メッセージとトピック指定に使用する文字列について

	payload は JSON 形式の文字列で記述する。

	payload中の JSON 文字列に日本語も使用可能であるが、この場合には UTF-8 で記述すること。

	payload中の JSON フォーマットの内容は一部のタグをデフォルトで規定("command", "result", "error_text") している以外は、
	アプリケーション側で自由に決めることができる。

	payload に格納する JSON 文字列は最大で 4096バイト。
	MGCP デバイス側でのリソース制限から、これよりも少ないバイト数の payload しか受け付けない場合がある。

	node 名は 50 バイト以内の任意の英数字を使用する。（日本語は使用しない）

	MQTT PUBLISH パケット送信時に指定する topic 文字列は 256 バイト未満に収まるように作成する。


(2) リクエストメッセージ(リプライメッセージ必要無しの場合）

	topic:
			* 全てのノードにリクエストメッセージを送信する
				/request/_ALL_/<node_name_from>

			* 特定のノードにリクエストメッセージを送信する
				/request/<node_name_to>/<node_name_from>

	payload (json string):

			{"command":"xxxxxx", ... }

			"command" タグ(オプション):リクエスト内容(コマンド)を表す文字列を格納する。
			コマンドの種類にによって複数のパラメータをタグに含める場合がある。

			その他のタグはアプリケーション毎に決めた任意の内容を格納する。

	qos:
			0 or 1 or 2

	topicを構成する文字列の説明:

			"/reuqest"				リクエストメッセージを示す文字列（固定)
			"_ALL_"					全てのデバイス側で購読している事を期待したトピック名。(固定)
			<node_name_to>			特定のノードに送信する場合の送信先ノード名。デバイス側では常に自身のノード名のトピック
									を購読している事を期待する。
			<node_name_from>		リクエストメッセージを送信したデバイスのノード名

(3) リクエストメッセージ(リプライメッセージを要求する場合)

	topic:
			* 特定のノードにリクエストメッセージを送信する
				/request/<node_name_to>/<node_name_from>/<request_id>

			(*) 全てのノードに同時にリプライが必要なリクエストメッセージ
			(<node_name_to> に "_ALL"を指定)を送信することは推奨しない。

			ただ、<node_name_to> に指定したノード名で購読しているデバイスが複数存在していた場合には、
			結果的に複数のデバイスにリクエストをすることになる。この場合には、リクエスト元は最初にリプライを受信した
			内容をリクエスト結果として処理することになる。複数のリプライメッセージを同一の <request_id> で受信した
			場合には最初に受信したものだけを処理して、その他のメッセージは無視するべきである。

	payload (json string):

			{"command":"xxxxxx", ... }

			"command" タグ(オプション):リクエスト内容(コマンド)を表す文字列を格納する。
			コマンドの種類にによって複数のパラメータをタグに含める場合がある。

			その他のタグはアプリケーション毎に決めた任意の内容を格納する。

	qos:
			0 or 1 or 2

	topicを構成する文字列の説明:

			"/reuqest"				リクエストメッセージを示す文字列(固定)
			<node_name_to>			特定のノードに送信する場合の送信先ノード名。
									デバイス側では常に自身のノード名のトピックを購読している事を期待する。
			<node_name_from>		リクエストメッセージを送信したデバイスのノード名
			<request_id>			リクエストメッセージを特定するための任意の英数文字列。
									文字列長は自由で、リクエストメッセージ毎にユニークな内容にすること。

(4) リプライメッセージ

	topic:
			* リクエストメッセージに対応するリプライメッセージを送信する
				/reply/<node_name_to>/<node_name_from>/<request_id>

	payload (json string, < 1024 bytes):


			{"result":<"success"|"fail">,"error_text":<error_message>,.... }

			"result" タグ(オプション): リクエストの処理が正常終了した場合には"success" が設定され、
			エラー発生時には "fail" が設定される。

			"error_text" タグ(オプション): リクエスト処理中のエラー内容が格納される。エラーがない場合には空文字列 "" 
			を格納するか、または "error_text" タグごと省略可能。

			その他のタグはアプリケーション毎に決めた任意の内容を格納できる

	qos:
			0 or 1 or 2

	topicを構成する文字列の説明:

			"/reply"				リプライメッセージを示す文字列(固定)
			<node_name_to>			リプライメッセージ送信先ノード名。
									リクエストメッセージを受信した時の <node_name_from> と同じ内容を設定する。
									リクエスト発行元のデバイス側では、常に自身のノード名のトピックを購読している事を期待する。
			<node_name_from>		リプライメッセージの送信元ノード名。自身のノード名を設定する。
			<request_id>			リクエストメッセージで指定された <request_id> と同じ文字列を設定する。

(5) イベントメッセージ

	topic:
			* 全てのノードにイベントを送信する
				/event/_ALL_/<node_name_from>/<application_name>/<event_type_string>

			* 特定のノードにイベントを送信
				/event/<node_name_to>/<node_name_from>/<application_name>/<event_type_string>

	payload (json string):

			{ .... }

			アプリケーション毎に決めた任意の内容を格納する。

	qos:
			0 or 1 or 2

	topicを構成する文字列の説明:

			"/event"				イベントメッセージを示す文字列(固定)
			"_ALL_"					全てのデバイス側で購読している事を期待したトピック名。(固定)
			<node_name_to>			特定のノードに送信する場合の送信先ノード名。
									受信するデバイス側では常に自身のノード名のトピックを購読している事を期待する。
			<node_name_from>		イベントを送信したデバイスのノード名
			<application_name>		送信元のデバイスで動作しているアプリケーションの名前(任意の文字列)を格納する。
			<event_type_string>		イベントの種類を示す名前。イベント発生側で定義した任意のイベント名を格納する。



abs_agent		All Blue System

]]



--[[

●機能概要

MGCP ノード名からエンドポイント名(Title または ClientID) を取得する。

各MGCP デバイスから定期的に送信されている heartbeat イベントを利用して、
送信元のエンドポイントを検索する。

●関数定義

stat [,clientid] = mgcp_find_endpoint(node)

●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------

node:string		検索対象の MGCPデバイスノード名							"ESP32Node1"


●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean	エンドポイントが見つかった場合には true が入り、clientid に検索結果
				が格納される。見つからなかった場合には false が返ります。

				複数のエンドポイントから同一のノード名のデバイスが見つかった場合には
				最初に見つけたエンドポイントが返ります。
																		true

clientid:string	MQTT ブローカのエンドポイントを定義した ClientID 文字列または
				Title 文字列が返る
																		"リモートデバイス処理"

												abs_agent copyright(c)2018  All Blue System
]]

function mgcp_find_endpoint(...)
	local logmod = "mgcp_find_endpoint()"
	local argv = {...}
	local argc = #argv
	local search_node = ""
	local find_endpoint = ""

	------------------------------------------------------------------------------------------------
	-- パラメータ設定
	------------------------------------------------------------------------------------------------
	if argc < 1 then
		return false
	end

	if type(argv[1]) == "string" then
		search_node = argv[1]
	else
		return false
	end
	
	if search_node == "" then
		return false
	end

	------------------------------------------------------------------------------------------------
	-- Node からheartbeat を受信しているエンドポイント一覧を取得
	------------------------------------------------------------------------------------------------
	local stat,endpoint_arr = get_shared_strlist('$MGCP_EndPointList')
	if not stat then
		log_msg("*ERROR* get_shared_strlist() failed",logmod)
		return false
	end
	if #endpoint_arr == 0 then
		return false
	end

	------------------------------------------------------------------------------------------------
	-- 各エンドポイントに属しているノード名リストに入っているか?
	------------------------------------------------------------------------------------------------
	for k,v in ipairs(endpoint_arr) do
		if find_shared_strlist('$MGCP_Node_' .. v,search_node) then
			find_endpoint = v
			break
		end
	end

	if find_endpoint == "" then
		return false
	else
		return true,find_endpoint
	end

end

--[[

●機能概要

MGCP プロトコルを使用して、リクエストメッセージを MQTT ブローカ経由で送信してリプライを受信する。

●関数定義

stat [,reply] = mgcp_command(node, request [,noreply [,timeout] ] )


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
node:string		リクエストコマンドの送信対象のノード名						"ESP32Node1"

request:string	リクエストコマンド(JSONフォーマット) 文字列					{"command":"version"}
				ここで指定するコマンドは、送信対象ノード側で提供しているコマンドを指定すること。
																	
noreply:boolean	リプライメッセージを受信しないことを指定する。true または false を指定。
				パラメータ省略時は false でリプライ受信まで内部で wait する。
																			false

timeout:number	リプライメッセージ受信までのタイムアウト(ms) を指定する。
				パラメータ省略時は 2000ms(2秒) が指定されます。				6000

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean	MGCP リクエストの送信とリプライ受信処理が成功した場合には true が入る。
				パラメータ指定が間違っている場合や、リプライ受信時にタイムアウト
				した場合には false が返ります。

				指定したサービスが提供されていない等で、サーバー側のコマンド処理が失敗した
				場合でも、リプライメッセージを受信した時点で true が返る事に注意してください。

				サーバー側のコマンド処理が正常に終了したかどうかは、reply で返される JSON 文字列中の
				の "result" タグの値が "success" になっているかで判断できます。
																		true

reply:string	サーバーから返されたリプライメッセージ(JSON フォーマット)

							{"result":"success","error_text":"","version":"1.0.0"}


●更新履歴

2019/12/28	リプライ受信失敗時に node 情報を削除していたのを止めた。"agent_mgcp -c" コマンドで同等の動作が可能

2019/1/12 	ノードに対するコマンド送信失敗時に、該当するノードリストとエンドポイントリストからノードを削除するようにした。
			ノードから heartbeat イベントが正常に届いた場合には再びノードが再登録されるので削除しても問題はない。

abs_agent copyright(c)2017  All Blue System
]]

function mgcp_command(...)
	local logmod = "mgcp_command()"
	local timeout = 2000
	local noreply = false
	local clientid = ""
	local node = ""
	local request = ""
	local argv = {...}
	local argc = #argv
	local stat
	local topic = ""
	local request_id = ""
	local endpoint_arr

	------------------------------------------------------------------------------------------------
	-- パラメータ設定
	------------------------------------------------------------------------------------------------
	if argc < 2 then
		return false
	end

	if type(argv[1]) == "string" then
		node = argv[1]
	else
		return false
	end

	if type(argv[2]) == "string" then
		request = argv[2]
	else
		return false
	end

	if (argc >= 3) and (type(argv[3]) == "boolean") then
		noreply = argv[3]
	end

	if (argc == 4) and (type(argv[4]) == "number") then
		timeout = argv[4]
	end

	if node == "_ALL_" then
		------------------------------------------------------------------------------------------------
		-- heartbeat を受信しているエンドポイント一覧を取得
		------------------------------------------------------------------------------------------------
		stat,endpoint_arr = get_shared_strlist('$MGCP_EndPointList')
		if not stat then
			log_msg("*ERROR* get_shared_strlist() failed",logmod)
			return false
		end
		if #endpoint_arr == 0 then
			return false
		end

		------------------------------------------------------------------------------------------------
		-- 全てのエンドポイントに対してメッセージを送信
		------------------------------------------------------------------------------------------------
		for k,v in ipairs(endpoint_arr) do
			request_id = "REQ_" .. tostring(math.random(100000))
			topic = "/request/" .. node .. "/" .. g_hostname
			if not mqtt_publish(v,topic,request,0) then
				log_msg("*ERROR* mqtt_publish() failed",logmod)
				return false
			end
		end

		return true

	else
		------------------------------------------------------------------------------------------------
		-- ノード名から送信先のエンドポイントを検索
		------------------------------------------------------------------------------------------------
		stat,clientid = mgcp_find_endpoint(node)
		if not stat then
			log_msg("*ERROR* mgcp_find_endpoint() failed, node = " .. node,logmod)
			return false
		end
	
		------------------------------------------------------------------------------------------------
		-- リクエスト毎にユニークな識別子を作成
		------------------------------------------------------------------------------------------------
		request_id = "REQ_" .. tostring(math.random(100000))
	
		------------------------------------------------------------------------------------------------
		-- MQTT ブローカからリプライメッセージを受信した時に、自身のノードでリクエストした事を
		-- 判断できるように、グローバル共有変数 request_id の名前で作成したリクエストフラグに "1" を設定
		------------------------------------------------------------------------------------------------
		if not noreply then
			if not set_shared_data(request_id,"1",true) then
				log_msg("*ERROR* set_shared_data() failed",logmod)
				return false
			end
		end
	
		------------------------------------------------------------------------------------------------
		-- MQTT ブローカ経由でリクエストコマンドをサービス提供側ノードに配信
		-- ブローカに接続している node で指定されたデバイスが処理を行うことを期待する
		------------------------------------------------------------------------------------------------
		if noreply then
			topic = "/request/" .. node .. "/" .. g_hostname
		else
			topic = "/request/" .. node .. "/" .. g_hostname .. "/" .. request_id
		end
		if not mqtt_publish(clientid,topic,request,0) then
			log_msg("*ERROR* mqtt_publish() failed",logmod)
			remove_shared_strlist('$MGCP_Node_' .. clientid,node)
			remove_shared_strlist('$MGCP_NodeList',node)
			return false
		end
		
		if not noreply then
			------------------------------------------------------------------------------------------------
			-- サービス提供側ノードでコマンド処理が行われた後、送信されてくるリプライメッセージを待つ。
			-- MQTT ブローカから受信したリプライメッセージは、グローバル共有変数に設定された後に
			-- イベントがセットされる。この処理は MQTT_PUBLISH イベントハンドラ中に記述されている。
			------------------------------------------------------------------------------------------------
			--if not event_wait2(request_id,timeout) then
			if not event_wait2x(request_id,timeout) then
				set_shared_data(request_id,"",true) -- 不要になったリクエストフラグを削除する
				log_msg("*ERROR* event_wait2() timeout, name = " .. request_id,logmod)
				--remove_shared_strlist('$MGCP_Node_' .. clientid,node)
				--remove_shared_strlist('$MGCP_NodeList',node)
				return false
			end
			
			------------------------------------------------------------------------------------------------
			-- グローバル共有変数に設定されたリプライデータ取得と削除
			------------------------------------------------------------------------------------------------
			local stat,reply_data = get_shared_data("_" .. request_id,true)
			if not stat then
				log_msg("*ERROR* get_shared_data() failed",logmod)
				return false
			end
			return true,reply_data
		else
			return true
		end

	end

end

--[[

●機能概要

MGCP プロトコルを使用して、イベントメッセージを MQTT ブローカ経由で送信する。

●関数定義

stat = mgcp_event(node, type, event [,application] )


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
node:string		イベントメッセージ送信先のノード名						"ESP32Node1"
				全てのノードを対象にする場合には "_ALL_" を　node に指定する
																	
type:string		イベントメッセージ送信時のトピック文字列中の <event_type_string>
				を指定する。											"gpio_change_detect"

event:string	イベントデータ(JSONフォーマット) 文字列					{"gpio":23,"direction":"rise"}

application:string
				イベントメッセージ送信時のトピック文字列中の <application_name>
				を指定する。											"device_controller_2"
				省略時は "abs_agent" が使用される。

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean	MGCP イベントデータ送信が成功した場合には true が入る。
				パラメータ指定が間違っている場合や、リプライ受信時にタイムアウト
				した場合には false が返ります。
																		true

●更新履歴

2019/1/12 	ノードに対するイベント送信失敗時に、該当するノードリストとエンドポイントリストからノードを削除するようにした。
			ノードから heartbeat イベントが正常に届いた場合には再びノードが再登録されるので削除しても問題はない。

abs_agent copyright(c)2017  All Blue System
]]

function mgcp_event(...)
	local logmod = "mqtt_mgcp_event()"
	local clientid = ""
	local node = ""
	local event = ""
	local type_str = ""
	local topic = ""
	local application = "abs_agent"
	local argv = {...}
	local argc = #argv
	local endpoint_arr
	local stat

	------------------------------------------------------------------------------------------------
	-- パラメータ設定
	------------------------------------------------------------------------------------------------
	if argc < 3 then
		return false
	end

	if type(argv[1]) == "string" then
		node = argv[1]
	else
		return false
	end

	if type(argv[2]) == "string" then
		type_str = argv[2]
	else
		return false
	end

	if type(argv[3]) == "string" then
		event = argv[3]
	else
		return false
	end

	if (argc == 4) and (type(argv[4]) == "string") then
		application = argv[4]
	end


	if node == "_ALL_" then
		------------------------------------------------------------------------------------------------
		-- heartbeat を受信しているエンドポイント一覧を取得
		------------------------------------------------------------------------------------------------
		stat,endpoint_arr = get_shared_strlist('$MGCP_EndPointList')
		if not stat then
			log_msg("*ERROR* get_shared_strlist() failed",logmod)
			return false
		end
		if #endpoint_arr == 0 then
			return false
		end

		------------------------------------------------------------------------------------------------
		-- 全てのエンドポイントに対してメッセージを送信
		------------------------------------------------------------------------------------------------
		topic = "/event/" .. node .. "/" .. g_hostname .. "/" .. application .. "/" .. type_str
		for k,v in ipairs(endpoint_arr) do
			if not mqtt_publish(v,topic,event,0) then
				log_msg("*ERROR* mqtt_publish() failed",logmod)
				return false
			end
		end

		return true

	else
		------------------------------------------------------------------------------------------------
		-- ノード名から送信先のエンドポイントを検索
		------------------------------------------------------------------------------------------------
		stat,clientid = mgcp_find_endpoint(node)
		if not stat then
			log_msg("*ERROR* mgcp_find_endpoint() failed, node = " .. node,logmod)
			return false
		end

		------------------------------------------------------------------------------------------------
		-- MQTT ブローカ経由でイベントデータをノードに配信
		-- ブローカに接続している node で指定されたデバイスが処理を行うことを期待する
		------------------------------------------------------------------------------------------------
		topic = "/event/" .. node .. "/" .. g_hostname .. "/" .. application .. "/" .. type_str
	
		if not mqtt_publish(clientid,topic,event,0) then
			log_msg("*ERROR* mqtt_publish() failed",logmod)
			remove_shared_strlist('$MGCP_Node_' .. clientid,node)
			remove_shared_strlist('$MGCP_NodeList',node)
			return false
		end

		return true

	end

end



