--
-- XBEE_API_LIB.lua    ver1.00     2019/6/30
-- 
-- abs_agent  copyright(c) 2019 All Blue System
--

--[[

●機能概要

req[] に格納された XBee API フレーム配列のチェックサム値を計算する。
req[] には start delimiter を含むデータから、チェックサム部分を含まない直前までのデータを格納すること

]]
function xbee_calc_checksum(req)
	local sum = 0
	for i = 4,#req,1 do
		sum = sum + req[i]
	end
	return 0xFF - bit_and(0xFF,sum)
end

--[[

●機能概要

XBee 802.15.4 (S1) モジュールに AT Command フレームを送信してレスポンスフレームを受信する。
コマンド実行対象の XBee モジュールは API Mode1 で動作していること。

●関数定義

stat, frame [,mframe] = xbee_at_local(com, atcmd [,param])

●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"
atcmd:string	2 ASCII character(upper case)
		XBee API コマンドで指定可能な ATコマンド文字列。ASCII 大文字で２文字
																	"NI"
param:string	(hexstr)
		XBee API コマンドパラメータを指定する場合に、データ(複数バイト)を16進数文字列で指定する。
		パラメータ省略時や "" 空文字列を指定すると、コマンドパラメータは送信されない。

																	"44657669636535"
●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		コマンド送信とレスポンス受信が正常に行われた場合には true が返りエラー発生時は false が返る。
		コマンド実行ステータスが成功しているかどうかは frame 中の Status バイトを確認すること。

frame:string	(hexstr)
		コマンド実行後に受信したレスポンスフレームが 16進数文字列で格納される。
												"7E000C88524E49004465766963653509"

mframe:Table [1..max_multi_frame#] of string	(hexstr)
		マルチフレームを構成するレスポンスフレームを受信した場合に、マルチフレーム部分のデータが
		16進数文字列の配列に格納される。mframe にはマルチフレーム部分のみが格納されて、
		実行結果ステータスを含むレスポンスフレームは同時に frame に返る。

				mframe[1] "7E001888524E44000B020013A200404AC3973E44657669636532002D"
				mframe[2] "7E001888524E44000D040013A20040558026454465766963653400C9"
				mframe[3] "7E001888524E44000C030013A200404AC3983A44657669636533002D"
				mframe[4] "7E001888524E44000A010013A200404AC39C3B44657669636531002E"


●更新履歴

2019/5/17 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_at_local(...)
	local argv = {...}
	local argc = #argv
	local req = {}
	local atcmd
	local param = {}
	local data_size
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if (argc < 2) or (argc > 3) then
		return false
	end

	atcmd = str_to_tbl(argv[2])
	if #atcmd ~= 2 then
		return false
	end

	if (argc >= 3) then
		param = hex_to_tbl(argv[3])
	end
	data_size = 4 + #param

	------------------------------------------------------------------------------------------------
	-- リクエストフレーム組み立て
	------------------------------------------------------------------------------------------------
	table.insert(req,0x7E)
	table.insert(req,bit_and(0xFF,bit_rshift(data_size,8)))
	table.insert(req,bit_and(0xFF,data_size))
	table.insert(req,0x08)					-- API_ID
	table.insert(req,math.random(1,0xFE))	-- Frame_ID
	table.insert(req,atcmd[1])
	table.insert(req,atcmd[2])
	for k,v in ipairs(param) do
		table.insert(req,v)
	end
	table.insert(req,xbee_calc_checksum(req))

	------------------------------------------------------------------------------------------------
	-- XBee APIリクエストフレーム送信とリプライフレーム受信
	------------------------------------------------------------------------------------------------
	return serial_write(argv[1],tbl_to_hex(req))
end

--[[

●機能概要

XBee 802.15.4 (S1) モジュールに NodeDiscover コマンドを実行した後、受信したフレームデータを整形して
発見した各デバイス情報を配列で返す。

●関数定義

stat, addr64, addr16, node_id, rssi = xbee_discover(com)

●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		コマンド送信とレスポンス受信が正常に行われた場合には true が返りエラー発生時は false が返る。
		コマンド実行ステータスが成功しているかどうかは frame 中の Status バイトを確認すること。

addr64:Table [1..maxnode#] of string	(hexstr)
		発見したノードの SerialNumber(64bit address) 配列。
		各ノードのアドレスは16進数文字列で格納される。

addr16:Table [1..maxnode#] of string	(hexstr)
		発見したノードの 16bit address配列。
		各ノードのアドレスは16進数文字列で格納される。

node_id:Table [1..maxnode#] of string
		発見したノードの NodeIdentifier配列。

rssi:Table [1..maxnode#] of string
		発見したノードの RSSI 値配列。
		各ノードの RSSI 値は10進数文字列で格納される。

●更新履歴

2019/5/18 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_discover(...)
	local logmod = "xbee_discover()"
	local argv = {...}
	local argc = #argv
	local stat,frame,mframe
	local frame_arr,mframe_arr
	local addr16 = {}
	local addr64 = {}
	local node_id = {}
	local rssi = {}
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if argc ~= 1 then
		return false
	end

	------------------------------------------------------------------------------------------------
	-- NDコマンド実行
	------------------------------------------------------------------------------------------------
	stat,frame,mframe = xbee_at_local(argv[1],"ND")
	if not stat then return false end
	frame_arr = hex_to_tbl(frame)
	if frame_arr[8] ~= 0x00 then -- check AT-response "Status" byte
		log_msg("*ERROR* NodeDiscover failed.",logmod)
		return false
	end

	------------------------------------------------------------------------------------------------
	-- ノードリストを検索してリターン値テーブルを作成
	------------------------------------------------------------------------------------------------
	for k,v in ipairs(mframe) do
		local arr = {}
		mframe_arr = hex_to_tbl(v)
		table.insert(addr16,string.sub(v,17,20))
		table.insert(addr64,string.sub(v,21,36))
		table.insert(rssi,tostring(tonumber("0x" .. string.sub(v,37,38))))
		for i = 20,#mframe_arr,1 do
			if mframe_arr[i] == 0x0 then break end
			table.insert(arr,mframe_arr[i])
		end
		table.insert(node_id,tbl_to_str(arr))
	end

	return true,addr64,addr16,node_id,rssi 

end

--[[

●機能概要

XBee 802.15.4 (S1) モジュールに NodeDiscover コマンドを実行した後、受信したフレームデータを
元に XBeeDevice マスターファイルを作成・更新する。

abs_agent に接続しているローカルの XBee デバイスについても AT コマンドで受信した情報を元に
同様にマスターファイルに登録する。

マスター上の既存のデバイス情報は更新され、xbee_rebuild_master() コール時に見つからなかった
既存のデバイスはそのままマスターファイルに残されます。

●関数定義

stat = xbee_rebuild_master(com)


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		マスター更新が正常に行われた場合には true が返りエラー発生時は false が返る。


●更新履歴

2019/5/21 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_rebuild_master(...)
	local logmod = "xbee_rebuild_master()"
	local argv = {...}
	local argc = #argv
	local master = "XBeeDevice"
	local stat, addr64, addr16,node_id, rssi, frame ,mframe, frame_arr
	local SH, SL, MY, NI
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if argc ~= 1 then
		return false
	end

	------------------------------------------------------------------------------------------------
	-- MASTERS に XBeeDevice マスターが登録済みであるかを調べて、無ければ新規作成する
	------------------------------------------------------------------------------------------------
	if not master_exist(master) then
		if not master_create(master,"XBee device master") then error() end
	end

	------------------------------------------------------------------------------------------------
	-- NodeDiscover を実行して周辺のデバイスを検索する
	------------------------------------------------------------------------------------------------
	stat, addr64, addr16, node_id, rssi = xbee_discover(argv[1])
	if not stat then
		log_msg("*ERROR* xbee_discover() failed.",logmod)
		return false
	end

	------------------------------------------------------------------------------------------------
	-- NDコマンドの結果で得られたデバイスを XBeeDevice マスターに登録する
	------------------------------------------------------------------------------------------------
	for i,v in ipairs(addr64) do

		-- マスターに既存のDeviceレコードが存在している場合は削除する
		if not master_item_delete(master,addr64[i],true) then error() end

		-- マスターに新規デバイスを登録する
		if not master_item_add(master,list_to_csv("SerialNumber","MyAddress16","NodeIdentifier","RSSI"),
									list_to_csv(addr64[i],addr16[i],node_id[i],rssi[i])) then error() end
		-- log_msg("addr64: " .. addr64[i] .. " addr16: " .. addr16[i] .." node_id: " .. node_id[i] .." rssi: " .. rssi[i],g_script)
	end

	------------------------------------------------------------------------------------------------
	-- abs_agent に接続している XBee モジュールもマスターに登録する。
	-- NodeDiscover では取得できないので、シリアル番号、16ビットアドレス、NodeID について ATコマンドで
	-- 問い合わせる
	------------------------------------------------------------------------------------------------

	------------------------------------------------------------------------------------------------
	-- SHコマンド実行
	------------------------------------------------------------------------------------------------
	stat,frame,mframe = xbee_at_local(argv[1],"SH")
	if not stat then return false end
	frame_arr = hex_to_tbl(frame)
	if frame_arr[8] == 0x00 then
		SH = string.sub(frame,17,24)
	else
		log_msg("*ERROR* SH failed.",logmod)
		return false
	end

	------------------------------------------------------------------------------------------------
	-- SLコマンド実行
	------------------------------------------------------------------------------------------------
	stat,frame,mframe = xbee_at_local(argv[1],"SL")
	if not stat then return false end
	frame_arr = hex_to_tbl(frame)
	if frame_arr[8] == 0x00 then
		SL = string.sub(frame,17,24)
	else
		log_msg("*ERROR* SL failed.",logmod)
		return false
	end

	------------------------------------------------------------------------------------------------
	-- MYコマンド実行
	------------------------------------------------------------------------------------------------
	stat,frame,mframe = xbee_at_local(argv[1],"MY")
	if not stat then return false end
	frame_arr = hex_to_tbl(frame)
	if frame_arr[8] == 0x00 then
		MY = string.sub(frame,17,20)
	else
		log_msg("*ERROR* MY failed.",logmod)
		return false
	end

	------------------------------------------------------------------------------------------------
	-- NIコマンド実行
	------------------------------------------------------------------------------------------------
	stat,frame,mframe = xbee_at_local(argv[1],"NI")
	if not stat then return false end
	frame_arr = hex_to_tbl(frame)
	if frame_arr[8] == 0x00 then
		local arr = {}
		for i = 9,#frame_arr - 1,1 do
			if frame_arr[i] == 0x0 then break end
			table.insert(arr,frame_arr[i])
		end
		NI = tbl_to_str(arr)
	else
		log_msg("*ERROR* NI failed.",logmod)
		return false
	end

	------------------------------------------------------------------------------------------------
	-- マスターに既存のローカルデバイスレコードが存在している場合は削除する
	------------------------------------------------------------------------------------------------
	if not master_item_delete(master,SH .. SL,true) then error() end

	------------------------------------------------------------------------------------------------
	-- マスターにローカルデバイスを登録する
	------------------------------------------------------------------------------------------------
	if not master_item_add(master,list_to_csv("SerialNumber","MyAddress16","NodeIdentifier","RSSI"),
										list_to_csv(SH .. SL,MY,NI,"")) then error() end

	return true

end

--[[

●機能概要

パラメータに指定された XBeeデバイスの 16bit address, 64bit address, NodeIdentifier文字列と
そのデバイスが abs_agent にシリアル接続されたローカルデバイスであるかを示すフラグを返す。

xbee_rebuid_master() でマスターに登録済みの XBee デバイスリストを検索して該当するデバイス情
報を取得します。dev に指定した文字列は、マスター中の NodeIdentifierに一致するものがあるかど
うかを先に検索します。一致するものが見つからなかった場合には、デバイスアドレス(16bit,64bit)
を順に検索します。

●関数定義

stat, addr64, addr16, node_id, is_local = xbee_find_device(com, dev)


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"
dev:string
		検索対象のXBeeデバイスを指定する。文字列には以下のいずれかを指定する。

			XBeeデバイスの Node Identification 文字列
			XBeeデバイスの MyAddress(16bitアドレス)16進数表記の文字列
			XBeeデバイスの SerialNumber(64bitアドレス)16進数表記の文字列
																		"Device5"

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		指定したデバイスが見つかった場合には true が返り、
		見つからなかった場合やエラー発生時は false が返る。
																		true

addr64:string	(hexstr)
		発見したノードの SerialNumber(64bit address)。16進数文字列で格納される。
																"0013A200409FD39D"
addr16:string	(hexstr)
		発見したノードの 16bit address。16進数文字列で格納される。
																"0F05"
node_id:string
		発見したノードの NodeIdentifier文字列。
																"Device5"
is_local:boolean
		XBeeデバイスが abs_agent のCOM ポートに接続されたローカルデバイス
		であるかどうかを示す
																true

●更新履歴

2019/5/22 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

-- stat, addr64 = xbee_get_local_addr64(com)
-- COMポートに接続された XBee デバイスの64ビットアドレスを取得
function xbee_get_local_addr64(com)
	local logmod = "xbee_get_local_addr64()"
	local addr64
	local com_port
	local stat, frame ,mframe, frame_arr
	local SH,SL

	------------------------------------------------------------------------------------------------
	-- 関数実行時に、共有データ $XBEE_LOCAL_ADDR64_<ComPort> がまだセットされていない場合はセットする。
	------------------------------------------------------------------------------------------------
	stat, com_port = serial_find_device(com)
	if not stat then
		log_msg("*ERROR* serial_find_device() failed.",logmod)
		return false
	end
	stat, addr64 = get_shared_data("$XBEE_LOCAL_ADDR64_" .. com_port)
	if addr64 == "" then

		------------------------------------------------------------------------------------------------
		-- SHコマンド実行
		------------------------------------------------------------------------------------------------
		stat,frame,mframe = xbee_at_local(com,"SH")
		if not stat then return false end
		frame_arr = hex_to_tbl(frame)
		if frame_arr[8] == 0x00 then
			SH = string.sub(frame,17,24)
		else
			log_msg("*ERROR* SH failed.",logmod)
			return false
		end
	
		------------------------------------------------------------------------------------------------
		-- SLコマンド実行
		------------------------------------------------------------------------------------------------
		stat,frame,mframe = xbee_at_local(com,"SL")
		if not stat then return false end
		frame_arr = hex_to_tbl(frame)
		if frame_arr[8] == 0x00 then
			SL = string.sub(frame,17,24)
		else
			log_msg("*ERROR* SL failed.",logmod)
			return false
		end

		addr64 = SH .. SL
		if not set_shared_data("$XBEE_LOCAL_ADDR64_" .. com_port,addr64) then error() end
	end

	return true, addr64

end

function xbee_find_device(...)
	local logmod = "xbee_find_device()"
	local argv = {...}
	local argc = #argv
	local master = "XBeeDevice"
	local stat
	local local_addr64
	local found_dev
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if argc ~= 2 then
		return false
	end

	--------------------------------------------------------------
	-- COMポートに接続された XBee デバイスの64ビットアドレスを取得
	--------------------------------------------------------------
	stat, local_addr64 = xbee_get_local_addr64(argv[1])
	if not stat then
		return false
	end

	--------------------------
	-- NodeIdentifier を検索
	--------------------------
	stat, found_dev = master_item_find(master,"NodeIdentifier",argv[2])

	--------------------------
	-- SerialNumber を検索
	--------------------------
	if not stat then
		stat, found_dev = master_item_find(master,"SerialNumber",argv[2])
	end

	--------------------------
	-- MyAddress16 を検索
	--------------------------
	if not stat then
		stat, found_dev = master_item_find(master,"MyAddress16",argv[2])
	end

	if not stat then
		log_msg("*ERROR* device not found, dev = " .. argv[2],logmod)
		return false
	end

	return true,found_dev["SerialNumber"],found_dev["MyAddress16"],found_dev["NodeIdentifier"],(found_dev["SerialNumber"] == local_addr64)

end

--[[

●機能概要

XBee 802.15.4 (S1) モジュールに Remote AT Command APIまたは(local) AT Command APIフレームを送信して
レスポンスフレームを受信する。コマンド実行対象の XBee モジュールとリモート側の XBee モジュールは
API Mode1 で動作していること。

●関数定義

stat, frame [,mframe] = xbee_at_command(com, dest, atcmd [,param [,option] ] )


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"
dest:string
		送信先のXBeeデバイスを指定する。dest文字列には以下のいずれかを指定する。
		XBeeデバイスの Node Identification 文字列
		XBeeデバイスの Serial Number(64bitアドレス)16進数表記の文字列
		XBeeデバイスの Source Address(16bitアドレス)16進数表記の文字列
		"*" ブロードキャストアドレス
		"" 空文字列は、abs_agent に接続したXBeeデバイスが対象

		"" (空文字列) を指定した場合にはローカルデバイスに対して(local)AT Command APIを
		実行する。16bit/64bitアドレスや NodeIdentifier で指定したデバイスがローカルデバイスだった場合にも同様に
		(local) AT Command API を使用する。それ以外のリモートデバイスに対しては Remote AT Command API を使用する。

																	"Device2"

atcmd:string	2 ASCII character(upper case)
		XBee API コマンドで指定可能な ATコマンド文字列。ASCII 大文字で２文字
																	"NI"
param:string	(hexstr)
		XBee API コマンドパラメータを指定する場合に、データ(複数バイト)を16進数文字列で指定する。
		パラメータ省略時や "" 空文字列を指定すると、コマンドパラメータは送信されない。

																	"44657669636535"
option:string	(hexstr)
		Remote AT Command API 中に指定する Option バイト(1byte)
		省略時は ApplyChange 0x02 が指定される。					"02"


●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		コマンド送信とレスポンス受信が正常に行われた場合には true が返りエラー発生時は false が返る。
		コマンド実行ステータスが成功しているかどうかは frame 中の Status バイトを確認すること。

frame:string	(hexstr)
		コマンド実行後に受信したレスポンスフレームが 16進数文字列で格納される。
												"7E000C88524E49004465766963653509"

mframe:Table [1..max_multi_frame#] of string	(hexstr)
		マルチフレームを構成するレスポンスフレームを受信した場合に、マルチフレーム部分のデータが
		16進数文字列の配列に格納される。mframe にはマルチフレーム部分のみが格納されて、
		実行結果ステータスを含むレスポンスフレームは同時に frame に返る。
		** 注意 ** 	Remote AT Command APIではマルチフレームを受信しないので常に nil になります

				mframe[1] "7E001888524E44000B020013A200404AC3973E44657669636532002D"
				mframe[2] "7E001888524E44000D040013A20040558026454465766963653400C9"
				mframe[3] "7E001888524E44000C030013A200404AC3983A44657669636533002D"
				mframe[4] "7E001888524E44000A010013A200404AC39C3B44657669636531002E"



●更新履歴

2019/5/25 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_at_command(...)
	local argv = {...}
	local argc = #argv
	local req = {}
	local atcmd
	local param = {}
	local option = {}
	local data_size
	local stat, addr64, addr16, node_id, is_local
	local addr64_arr
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if (argc < 3) or (argc > 5) then
		return false
	end

	if argv[2] == '' then -- local AT Command
		return xbee_at_local(argv[1],argv[3],argv[4])
	end

	if argv[2] == '*' then
		addr64 = "000000000000FFFF"
	else
		stat, addr64, addr16, node_id, is_local = xbee_find_device(argv[1],argv[2])
		if not stat then return false end
		if is_local then -- local AT Command
			return xbee_at_local(argv[1],argv[3],argv[4])
		end
	end
	addr64_arr = hex_to_tbl(addr64) 

	atcmd = str_to_tbl(argv[3])
	if #atcmd ~= 2 then
		return false
	end

	if (argc >= 4) then
		param = hex_to_tbl(argv[4])
	end

	if (argc == 5) then
		option = hex_to_tbl(argv[5])
	else
		table.insert(option,0x02) 			-- command option, default:0x02 (apply change)
	end

	data_size = 15 + #param

	------------------------------------------------------------------------------------------------
	-- リクエストフレーム組み立て
	------------------------------------------------------------------------------------------------
	table.insert(req,0x7E)
	table.insert(req,bit_and(0xFF,bit_rshift(data_size,8)))
	table.insert(req,bit_and(0xFF,data_size))
	table.insert(req,0x17)					-- API_ID
	table.insert(req,math.random(1,0xFE))	-- Frame_ID
	table.insert(req,addr64_arr[1])
	table.insert(req,addr64_arr[2])
	table.insert(req,addr64_arr[3])
	table.insert(req,addr64_arr[4])
	table.insert(req,addr64_arr[5])
	table.insert(req,addr64_arr[6])
	table.insert(req,addr64_arr[7])
	table.insert(req,addr64_arr[8])
	table.insert(req,0xFF)			-- 常に64bitアドレスでリモートAT コマンドを実行
	table.insert(req,0xFE)
	table.insert(req,option[1])
	table.insert(req,atcmd[1])
	table.insert(req,atcmd[2])
	for k,v in ipairs(param) do
		table.insert(req,v)
	end
	table.insert(req,xbee_calc_checksum(req))

	------------------------------------------------------------------------------------------------
	-- XBee APIリクエストフレーム送信とリプライフレーム受信
	------------------------------------------------------------------------------------------------
	return serial_write(argv[1],tbl_to_hex(req))
end


--[[

●機能概要

XBee 802.15.4 (S1) モジュールに TX Transmit Request APIフレームを送信してレスポンスフレームを受信する。
コマンド実行対象のローカル XBee モジュールと送信先となるリモート側の XBee モジュールは API Mode1 で動作していること。

●関数定義

stat, frame = xbee_transmit_data(com, dest, data [,option] ] )


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"
dest:string
		データ送信先のXBeeデバイスを指定する。dest文字列には以下のいずれかを指定する。
		XBeeデバイスの Node Identification 文字列
		XBeeデバイスの Serial Number(64bitアドレス)16進数表記の文字列
		XBeeデバイスの Source Address(16bitアドレス)16進数表記の文字列
		"*" ブロードキャストアドレス
																	"Device2"

data:string	(hexstr)
		送信データ(複数バイト)を16進数文字列で指定する。最大 100 バイトまで指定可。

																	"010203F1F2F3"
option:string	(hexstr)
		Transmit Request API 中に指定する Option バイト(1byte)
		省略時は 0x00 が指定される。								"04"


●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		コマンド送信とレスポンス受信が正常に行われた場合には true が返りエラー発生時は false が返る。
		コマンド実行ステータスが成功しているかどうかは frame 中の Status バイトを確認すること。

frame:string	(hexstr)
		コマンド実行後に受信した Transmit Statusフレームが 16進数文字列で格納される。
																	"7E000389C400B2"


●更新履歴

2019/5/26 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_transmit_data(...)
	local argv = {...}
	local argc = #argv
	local req = {}
	local data = {}
	local option = {}
	local data_size
	local stat, addr64, addr16, node_id, is_local
	local addr64_arr
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if (argc < 3) or (argc > 4) then
		return false
	end

	if argv[2] == '' then error() end

	if argv[2] == '*' then
		addr64 = "000000000000FFFF"
	else
		stat, addr64, addr16, node_id, is_local = xbee_find_device(argv[1],argv[2])
		if not stat then return false end
		if is_local then return false end
	end
	addr64_arr = hex_to_tbl(addr64) 

	if (argc >= 3) then
		data = hex_to_tbl(argv[3])
	end

	if (argc == 4) then
		option = hex_to_tbl(argv[4])
	else
		table.insert(option,0x00) 			-- command option, default:0x00
	end

	data_size = 11 + #data

	------------------------------------------------------------------------------------------------
	-- リクエストフレーム組み立て
	------------------------------------------------------------------------------------------------
	table.insert(req,0x7E)
	table.insert(req,bit_and(0xFF,bit_rshift(data_size,8)))
	table.insert(req,bit_and(0xFF,data_size))
	table.insert(req,0x00)					-- API_ID
	table.insert(req,math.random(1,0xFE))	-- Frame_ID
	table.insert(req,addr64_arr[1])
	table.insert(req,addr64_arr[2])
	table.insert(req,addr64_arr[3])
	table.insert(req,addr64_arr[4])
	table.insert(req,addr64_arr[5])
	table.insert(req,addr64_arr[6])
	table.insert(req,addr64_arr[7])
	table.insert(req,addr64_arr[8])
	table.insert(req,option[1])
	for k,v in ipairs(data) do
		table.insert(req,v)
	end
	table.insert(req,xbee_calc_checksum(req))

	------------------------------------------------------------------------------------------------
	-- XBee APIリクエストフレーム送信とリプライフレーム受信
	------------------------------------------------------------------------------------------------
	return serial_write(argv[1],tbl_to_hex(req))
end

--[[

●機能概要

指定したXBeeデバイスに接続している、TDCP モニタプログラムが動作しているコントローラデバイスに任意の
TDCPコマンドを送信する。

●関数定義

stat [,result] = xbee_tdcp_command(com, dest, tdcp_cmd [,no_result])


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"
dest:string
		データ送信先のXBeeデバイスを指定する。dest文字列には以下のいずれかを指定する。
		XBeeデバイスの Node Identification 文字列
		XBeeデバイスの Serial Number(64bitアドレス)16進数表記の文字列
		XBeeデバイスの Source Address(16bitアドレス)16進数表記の文字列
		"*" ブロードキャストアドレス
																	"Device2"

tdcp_cmd:string
		TDCP コマンド文字列。最大 100 バイトまで指定可。

																	"version"
no_result:boolean
		コマンドリプライデータ(result) の受信を省略する。
		パラメータ省略時には false を指定。							false


●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		コマンド送信とレスポンス受信が正常に行われた場合には true が返りエラー発生時は false が返る。
		コマンド実行ステータスが成功しているかどうかは frame 中の Status バイトを確認すること。

result:Table [1..#max_frame] of String
		コマンドリプライデータ文字列配列
		カンマ区切りの TDCP コマンドリプライデータを、配列に変換した値で取得する。


●更新履歴

2019/5/28 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_tdcp_command(...)
	local logmod = "xbee_tdcp_command()"
	local argv = {...}
	local argc = #argv
	local data_hexstr
	local no_result = false
	local prefix
	local stat, frame, reply_data
	local timeout = 2000
	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if (argc < 3) or (argc > 4) then
		return false
	end

	if argv[2] == '*' then
		no_result = true
	end

	if (argc == 4) and (type(argv[4]) == "boolean") then
		no_result = argv[4]
	end

	if no_result then
		prefix = "$$$"
	else
		prefix = "$$$" .. tostring(math.random(1,99999))
	end
	data_hexstr = tbl_to_hex(str_to_tbl(prefix .. "," .. argv[3]))

	------------------------------------------------------------------------------------------------
	-- TDCPリクエストパケット送信とリプライフレーム受信
	------------------------------------------------------------------------------------------------
	if no_result then
		stat, frame = xbee_transmit_data(argv[1],argv[2],data_hexstr)
		return stat
	else
		set_shared_data("$TDCP_STORE_REPLY_" .. prefix,"1",true)

		stat, frame = xbee_transmit_data(argv[1],argv[2],data_hexstr)
		if not stat then
			set_shared_data("$TDCP_STORE_REPLY_" .. prefix,"",true) -- 不要になったリクエストフラグを削除する
			return false
		end

		if not event_wait2x("$TDCP_COMMAND_" .. prefix,timeout) then
			set_shared_data("$TDCP_STORE_REPLY_" .. prefix,"",true) -- 不要になったリクエストフラグを削除する
			log_msg("*ERROR* event_wait2() timeout",logmod)
			return false
		end
		
		------------------------------------------------------------------------------------------------
		-- グローバル共有変数に設定されたリプライデータ取得と削除
		------------------------------------------------------------------------------------------------
		stat,reply_data = get_shared_data("$TDCP_" .. prefix,true)
		if not stat then
			log_msg("*ERROR* get_shared_data() fail",logmod)
			return false
		end
		return true,csv_to_tbl(reply_data)
	end

end


--[[

●機能概要

xbee_rebuild_master() ライブラリ関数によって、探索・登録された周辺の XBee デバイスリスト
を取得する。

xbee_all_list() では、周辺デバイスの探索や新規登録は行いません、これらの操作が必要な場合は、
xbee_rebuild_master() ライブラリ関数をコールしてください。

マスターに登録された XBeeデバイスの 16bit address, 64bit address, NodeIdentifier文字列と
そのデバイスが com パラメータで指定した、abs_agent にシリアル接続されたローカルデバイスで
あるかを示すフラグを返す。

●制限事項

abs_agent のシリアルポートに複数の(ローカル)XBee デバイスを登録して、それぞれ別の PAN_ID や同一の PAN_ID
を設定して、それぞれの XBee 経由でリモート XBeeデバイスを操作することができます。

このとき、xbee_rebuild_master() で登録されるデバイスマスターは共通のものを使用しています。
このため、xbee_all_list(com) をコールするときに指定した com デバイスに接続中の XBee の PAN_ID 以外の
デバイスもリターン値に返ります。

●関数定義

stat, addr64, addr16, node_id, is_local = xbee_all_list(com)


●リクエストパラメータ
---------------------------------------------------------------------------------
パラメータ名	説明	            									値の例
---------------------------------------------------------------------------------
com:string
		シリアルデバイスに "XBEE" タイプで登録済みの COMPort名またはタイトル名
																	"/dev/ttyUSB0"

●リターン値
---------------------------------------------------------------------------------
リターン値名	説明	            									値の例
---------------------------------------------------------------------------------
stat:boolean
		指定したデバイスが見つかった場合には true が返り、
		見つからなかった場合やエラー発生時は false が返る。
																		true

addr64:Table [1..#max_item] of String
		発見したノードの SerialNumber(64bit address)配列。16進数文字列で格納される。
																"0013A200409FD39D"
addr16:Table [1..#max_item] of String
		発見したノードの 16bit address配列。16進数文字列で格納される。
																"0F05"
node_id:Table [1..#max_item] of String
		発見したノードの NodeIdentifier文字列配列。
																"Device5"
is_local:Table [1..#max_item] of boolean
		XBeeデバイスが abs_agent のCOM ポートに接続されたローカルデバイス
		であるかどうかを示す
																true

●更新履歴

2019/6/12 	初版作成

abs_agent copyright(c) 2019  All Blue System

]]

function xbee_all_list(...)
	local argv = {...}
	local argc = #argv
	local master = "XBeeDevice"
	local stat
	local local_addr64
	local addr64_arr = {}
	local addr16_arr = {}
	local node_id_arr = {}
	local is_local_arr = {}

	------------------------------------------------------------------------------------------------
	-- パラメータチェック
	------------------------------------------------------------------------------------------------
	if argc ~= 1 then
		return false
	end

	--------------------------------------------------------------
	-- COMポートに接続された XBee デバイスの64ビットアドレスを取得
	--------------------------------------------------------------
	stat, local_addr64 = xbee_get_local_addr64(argv[1])
	if not stat then
		return false
	end

	--------------------------------------------------------------
	-- マスターファイルを検索してデバイスリストを取得
	--------------------------------------------------------------
	stat, addr64_arr, addr16_arr, node_id_arr = master_item_list(master)
	if not stat then
		return false
	end

	for k,v in ipairs(addr64_arr) do
		table.insert(is_local_arr,(v == local_addr64))
	end

	return true,addr64_arr,addr16_arr, node_id_arr,is_local_arr

end
