
--[[

GRAPHIC_PRINT2.lua

change history:

ver1.10 2021/5/31	半角・全角混在時のスクリーン幅チェックのバグ修正

ver1.01 2021/1/4	SJIS フォントに文字に無い文字は強制的に '?' に変更

ver1.00 2020/12/19	初版

abs_agent copyright(c) 2021 All Blue System

]]


--[[

****************************************************************************************************

LCD/OLED デバイスに指定したフォントで文字列を表示する。

関数定義
		graphic_print2(x, y, str [,font [,size [,sp_h] ] ])

パラメータ
		x:number(opt)		文字列の表示座標 (X軸) を指定する。nil 指定時は 0 (左端)が設定される

		y:number(opt)		文字列の表示座標 (Y軸) を指定する。nil 指定時は画面上端に表示するように自動設定される

							具体的なY軸座標はフォントサイズと画面サイズから自動で決められる
		str:string			表示する文字列

		font:string(opt)	グローバル共有エリアにロード済みの日本語フォントのチャンネル名
							font パラメータ省略時は "SHINONOME_FONTX"(16x16全角)が指定される。
							font パラメータで指定したフォントデータが ASCII の場合には str で指定された文字列は全て
							バイト単位の文字コードで表示する。
							font パラメータで指定したフォントデータが Shift_JIS の場合には全角フォントで１文字毎に表示する。
							このとき、font に指定したチャンネル名の先頭に "E_" を付加した名前の ASCII フォントが
							存在する場合には、ASCII 文字(0 .. 0xFE の文字コード)にはこの ASCIIフォントを使用して表示する。
							ASCII フォントが見つからなかった場合には、(同じ字形の)全角フォントで ASCII 文字を表示する。

		size:number(opt)	文字を size 倍に拡大して表示する	default:1

		sp_h:number(opt)	縦方向の文字列間隔					default:0

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function graphic_print2(...)
	local argv = {...}
	local argc = #argv
	local x0,y0
	local font = "SHINONOME_FONTX"
	local font_w,font_h,e_font_w,e_font_h
	local font_ascii
	local use_e_font = false
	local size = 1 -- 拡大倍率
	local sp_h = 0 -- 縦方向の文字間隔
	local processed = false
	if (argc < 3) or (argc > 6) then return false end
	if argc >= 5 then size = argv[5] end
	if argc == 6 then sp_h = argv[6] end
	local str = argv[3]

	------------------------------
	-- フォントパラメータチェック
	------------------------------
	fontx_try_init()
	if argc >= 4 then font = argv[4] end
	local stat,font_info = shmem_copy(font,14,3) -- font_info[1]:width, font_info[2]:height, font_info[3]:Code 0-ASCII 1-SJIS
	if not stat then return false end
	font_w = font_info[1]
	font_h = font_info[2]
	font_ascii = (font_info[3] == 0)

	---------------------------------------------------------------------------------
	-- font が SJIS フォントテーブルの場合には、別途 ASCII フォントの存在をチェック
	---------------------------------------------------------------------------------
	local e_font = "E_" .. font
	if (not font_ascii) and find_shared_strlist("$FONTX_LIST",e_font) then
		local stat,e_font_info = shmem_copy(e_font,14,3)
		if stat and (e_font_info[3] == 0) then
			use_e_font = true
			e_font_w = e_font_info[1]
			e_font_h = e_font_info[2]
		end
	else
		e_font = ""
	end
 
	---------------------
	-- 表示開始位置計算
	---------------------
	if (argc >= 3) and argv[1] then x0 = argv[1] end
	if (argc >= 3) and argv[2] then y0 = argv[2] end
	local scr_w, scr_h = stat_check(graphic_set_size())
	if not x0 then x0 = 0 end
	if not y0 then y0 = scr_h - (font_h * size) end
	if (x0 > scr_w) or (y0 < 0) then return true end

	---------------------------------------------------------------------------------------------------------
	-- 文字列を全てバイトコード(ASCII コード)として解釈して、ASCII フォントのみを使用して表示
	---------------------------------------------------------------------------------------------------------
	if font_ascii then
		processed = true
		local ch_arr = str_to_tbl(str)
		if #ch_arr == 0 then return false end
		for k,v in ipairs(ch_arr) do
			local gstat, fdata = shmem_get_fontx(font,v)
			if not gstat then goto NEXT_CHAR1 end
			if x0 >= scr_w - (font_w * size) then
				x0 = 0
				y0 = y0 - (sp_h + (font_h * size))
			end
			if y0 < 0 then break end
			graphic_draw_fontx(x0,y0,font_w,font_h,tbl_to_hex(fdata),size)
			x0 = x0 + (font_w * size)
::NEXT_CHAR1::
		end
	end

	---------------------------------------------------------------------------------------------------------
	-- 文字列を文字毎に分解した後、バイトコード(ASCII)文字については同じ字形のマルチバイト文字に変換した後、
	-- 全てSJIS フォントのみを使用して表示
	---------------------------------------------------------------------------------------------------------
	if (not font_ascii) and (not use_e_font) then
		processed = true
		local stat,ch_arr = str_to_tbl2(str,2,0x3f)
		if not stat then return false end
		if #ch_arr == 0 then return false end
		fontx_ascii_to_zenkaku(ch_arr)	-- ASCII コード部分を同じ字形の全角文字に変換
		for k,v in ipairs(ch_arr) do
			local gstat, fdata = shmem_get_fontx(font,v)
			if not gstat then goto NEXT_CHAR2 end
			if x0 >= scr_w - (font_w * size) then
				x0 = 0
				y0 = y0 - (sp_h + (font_h * size))
			end
			if y0 < 0 then break end
			graphic_draw_fontx(x0,y0,font_w,font_h,tbl_to_hex(fdata),size)
			x0 = x0 + (font_w * size)
::NEXT_CHAR2::
		end
	end

	---------------------------------------------------------------------------------------------------------
	-- マルチバイト文字には SJIS フォントを使用して ASCII 文字には半角フォントを使用して表示
	---------------------------------------------------------------------------------------------------------
	if (not font_ascii) and use_e_font then
		local gstat,fdata
		processed = true
		local stat,ch_arr = str_to_tbl2(str,2,0x3f)
		if not stat then return false end
		if #ch_arr == 0 then return false end
		for k,v in ipairs(ch_arr) do
			if v < 0xff then
				gstat, fdata = shmem_get_fontx(e_font,v)
				if not gstat then goto NEXT_CHAR3 end
				if x0 >= scr_w - (e_font_w * size) then
					x0 = 0
					y0 = y0 - (sp_h + (e_font_h * size))
				end
				if y0 < 0 then break end
				graphic_draw_fontx(x0,y0,e_font_w,e_font_h,tbl_to_hex(fdata),size)
				x0 = x0 + (e_font_w * size)
			else
				gstat, fdata = shmem_get_fontx(font,v)
				if not gstat then goto NEXT_CHAR3 end
				if x0 >= scr_w - (font_w * size) then
					x0 = 0
					y0 = y0 - (sp_h + (font_h * size))
				end
				if y0 < 0 then break end
				graphic_draw_fontx(x0,y0,font_w,font_h,tbl_to_hex(fdata),size)
				x0 = x0 + (font_w * size)
			end
::NEXT_CHAR3::
		end
	end

	return processed
end
