
--[[
****************************************************************************************************

●機能概要

MGCP デバイスに接続する I2C デバイス操作用の API ライブラリ

2019/10/5 	mgcp_i2c_read() added.

copyright(c) 2018-2019 All Blue System

****************************************************************************************************
]]

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスから指定したバイト数のデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)
		read_len:number	データ読み込みバイト数

リターン値
		stat:boolean	true 成功、false 失敗
		hexstr_rpl:string	I2C デバイスから読み込んだデータ(16進数文字列)。
							コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function mgcp_i2c_read(node,port,addr,read_len)
	local stat,reply = mgcp_command(node,'{"command":"i2c_read","port":' .. tostring(port).. ',"read_len":' .. tostring(read_len).. 
											',"addr":"' .. addr .. '"}',false,3000)
	if stat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true,rpl_json.data
		else
			return false
		end
	else
		return false
	end
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスにデータを書き込む
		
パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)
		hexstr:string	書き込むデータ(16進数文字列、1バイトあたり2桁)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function mgcp_i2c_write(node,port,addr,hexstr)
	return mgcp_command(node,'{"command":"i2c_write","port":' .. tostring(port).. 
											',"addr":"' .. addr .. '","data":"' .. hexstr .. '"}',false,3000)
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスに複数のデータ列を書き込む

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)
		hexstr_csv:string	書き込むデータ(16進数文字列、1バイトあたり2桁)をCSV形式で並べた文字列
							例： '"12","AB","001234"'   各ブロックの16進数データはダブルコートで囲む事
		delay:number	ブロック書き込みするときの各書き込みトランザクション間のディレイ時間(ms)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function mgcp_i2c_write_block(node,port,addr,hexstr_csv,delay)
	return mgcp_command(node,'{"command":"i2c_write_block","port":' .. tostring(port).. ',"delay":' .. tostring(delay).. 
											',"addr":"' .. addr .. '","data":[' .. hexstr_csv .. ']}',false,4000)
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスにデータを書き込んだ後、指定したバイト数のデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)
		hexstr:string	書き込むデータ(16進数文字列、1バイトあたり2桁)
		read_len:number	データ読み込みバイト数

リターン値
		stat:boolean	true 成功、false 失敗
		hexstr_rpl:string	I2C デバイスから読み込んだデータ(16進数文字列)。
							コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function mgcp_i2c_write_read(node,port,addr,hexstr,read_len)
	local stat,reply = mgcp_command(node,'{"command":"i2c_write","port":' .. tostring(port).. ',"read_len":' .. tostring(read_len).. 
											',"addr":"' .. addr .. '","data":"' .. hexstr .. '"}',false,3000)
	if stat then
		local rpl_json = g_json.decode(reply)
		if rpl_json.result == "success" then
			return true,rpl_json.data
		else
			return false
		end
	else
		return false
	end
end

