
--[[
****************************************************************************************************

●機能概要

MGCP デバイスに接続する I2C デバイス操作用の API ライブラリ
ダイレクトコマンドモードを使用して、MGCP コマンドに比べて高速に動作する。

2020/10/6 ver1.0 新規作成

copyright(c) 2020 All Blue System

****************************************************************************************************
]]

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスにデータを書き込んだ後、指定したバイト数のデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:number		I2Cスレーブアドレス
		data:table of number	書き込むデータ
		read_len:number	データ読み込みバイト数

リターン値
		stat:boolean	true 成功、false 失敗
		rdata:table of number		
							I2Cデバイスから読み込んだデータ
							コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function build_frame_i2c_write_read(arr,port,addr,data,read_len)
	table.insert(arr,0x26)
	dxcmd_param_add_byte(arr,port)
	dxcmd_param_add_byte(arr,addr)
	dxcmd_param_add_word(arr,read_len)
	dxcmd_param_add_word(arr,0) -- no delay
	for i,v in ipairs(data) do
		table.insert(arr,v)
	end
end

function dxcmd_i2c_write_read(node,port,addr,data,read_len)
	local frame = {}
	build_frame_i2c_write_read(frame,port,addr,data,read_len)
	return dxcmd_exec_reply_get(node,frame)
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスから指定したバイト数のデータ読み込みを行う

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:number		I2Cスレーブアドレス
		read_len:number	データ読み込みバイト数

リターン値
		stat:boolean	true 成功、false 失敗
		rdata:table of number		
							I2Cデバイスから読み込んだデータ
							コマンド処理が成功したときのみ値が返る。

****************************************************************************************************
]]

function build_frame_i2c_read(arr,port,addr,read_len)
	table.insert(arr,0x27)
	dxcmd_param_add_byte(arr,port)
	dxcmd_param_add_byte(arr,addr)
	dxcmd_param_add_word(arr,read_len)
end

function dxcmd_i2c_read(node,port,addr,read_len)
	local frame = {}
	build_frame_i2c_read(frame,port,addr,read_len)
	return dxcmd_exec_reply_get(node,frame)
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスにデータを書き込む
		
パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:number		I2Cスレーブアドレス
		data:table of number	書き込むデータ

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function build_frame_i2c_write(arr,port,addr,data)
	table.insert(arr,0x25)
	dxcmd_param_add_byte(arr,port)
	dxcmd_param_add_byte(arr,addr)
	for i,v in ipairs(data) do
		table.insert(arr,v)
	end
end

function dxcmd_i2c_write(node,port,addr,data)
	local frame = {}
	build_frame_i2c_write(frame,port,addr,data)
	return dxcmd_exec_stat_check(node,frame)
end

--[[
****************************************************************************************************

MGCP ノードに接続した I2C デバイスに複数のデータ列を書き込む

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:number		I2Cスレーブアドレス
		data_arr:table of table of number
						書き込むデータ配列を複数格納した配列を指定する。
		delay:number	ブロック書き込みするときの各書き込みトランザクション間のディレイ時間(ms)

リターン値
		stat:boolean	true 成功、false 失敗

****************************************************************************************************
]]

function dxcmd_i2c_write_block(node,port,addr,data_arr,delay)
	local item_frame
	local frame = {}
	table.insert(frame,0x0d)
	dxcmd_param_add_word(frame,delay)
	for i,v in ipairs(data_arr) do
		item_frame = {}
		build_frame_i2c_write(item_frame,port,addr,v)
		dxcmd_param_add_word(frame,#item_frame)
		dxcmd_param_append(frame,item_frame)
	end
	return dxcmd_exec_stat_check(node,frame)
end

-- 16進数文字列を CSV 形式で複数列挙した文字列を、数値配列が複数格納された配列に変換
-- mgcp_i2c_write_block() 関数コール時に使用する hexstr_csv パラメータに渡していたデータを
--  dxcmd_i2c_write_block() 関数の data_arr パラメータ形式に変換するときに使用できる。
function dxcmd_hexscv_to_arr(hexcsv)
	local data_arr = {}
	local tmp_arr = csv_to_tbl(hexcsv)
	for i,v in ipairs(tmp_arr) do
		local arr = hex_to_tbl(v)
		table.insert(data_arr,arr)
	end
	return data_arr
end
